//*************************************************************************
//                       kinda like drops from bubble                      
//*************************************************************************
// Written by : Vojvo & Gunny
// Language   : VisuC6 + DX6.1
// Version    : 0.99.4
// Last update: 08.12.99 
//*************************************************************************
// 0.99.4 - droplist pouziva pro surface get() misto load()

#include <stdio.h>
#include <stdlib.h>
#include "dx8graph.h"
#include "tex8manager.h"
#include "gonio.h"
#include "utils.h"
#include "application.h"
#include "drops.h"


//#define DROPTEST

#define WARPCNT 144


//#########################################################################
//--------------------------------- kapky  --------------------------------
//#########################################################################
BubbleDrop::BubbleDrop(Texture gfx,int ix,int iy,int vx,int vy,
                       BYTE color, int g, int s)
{
   mul_x = ix << SHIFT_MOVE;
   mul_y = iy << SHIFT_MOVE;
   if (color > MAX_DROP_COLOR)
      color = DROP_BLACK;
   frameRect.left = color * DROP_TILE;
   frameRect.right = frameRect.left + DROP_TILE;
   frameRect.top = 0;
   frameRect.bottom = DROP_TILE;
   movex=vx;
   movey=vy;
   grave=g;
   warp=0;
   warpcnt=72;
	size = s;
	if ((size<3) || (size>64)) size = 3;
   b_mminx = static_cast<DWORD>(PLAYFIELD_MIN_X) << SHIFT_MOVE;
   b_mmaxx = static_cast<DWORD>(PLAYFIELD_MAX_X - DROP_PAINT) << SHIFT_MOVE;
   b_mminy = static_cast<DWORD>(PLAYFIELD_MIN_Y) << SHIFT_MOVE;
   b_mmaxy = static_cast<DWORD>(PLAYFIELD_MAX_Y - DROP_PAINT) << SHIFT_MOVE;
   this->gfx = gfx;
}

//==========================================================================
void BubbleDrop::Paint()
{
   int xx = mul_x >> SHIFT_MOVE;
   int yy = mul_y >> SHIFT_MOVE;
	RECT r;
	r.left = xx;
	r.top = yy;
	r.right = xx + size;
	r.bottom = yy + size;
	r = dx8.Coord(r);

   dx8.BltSprite(&r, gfx, &frameRect, 0, 0xffffffff); // 0xa0ffffff
}

//=========================================================================
int BubbleDrop::Move()
{
   movey += grave;
   movex += warp;
   mul_x += movex;
   mul_y += movey;
   if (mul_x < b_mminx || mul_x > b_mmaxx || mul_y < b_mminy || mul_y > b_mmaxy)
   {
      return OUT_OF_BOUNDS;
   }
   ++warpcnt;
   if (warpcnt >= WARPCNT)
   {
      warpcnt = 0;
      warp = -warp;
   }
   return IN_BOUNDS;
}


//#########################################################################
//------------------------------ kapkecnik --------------------------------
//#########################################################################

DropList::DropList()
{ 
   count = 0; 
	gfx = NULL;
	textureManager->Load(&gfx,"hd-drops.r24", glSpritesFake);
}

//==========================================================================
void DropList::Blast(int ax, int ay, BYTE color, int grav)
//rozprsk bubliny na ax:ay (levy horni roh) a barvou color
{
   int tomake=(MAXDROP-count+4)/10;
   int angle;
   for (int i=0; i<tomake; i++)
   {
      angle= 220 + random(100);
		int t = 2*sinx(angle)-random(1000);
      list[count]=new BubbleDrop(gfx,ax,ay,2*cosx(angle),
				     t,color,grav, DROP_PAINT-random(2));
      count++;
   }
}

//==========================================================================
void DropList::HorizontalBlast(int ax, int ay, BYTE color, int grav, int direction)
//rozprsk od zdi na ax:ay (levy horni roh) a barvou color
{
   int tomake=(MAXDROP-count+4)/10;
   int angle;
   for (int i=0; i<tomake; i++)
   {
      angle= 220 + random(100);
      list[count]=new BubbleDrop(gfx,ax,ay,direction*2*sinx(angle),2*cosx(angle),color,grav, DROP_PAINT-random(2)-2);
      count++;
   }
}

//==========================================================================
void DropList::OneDrop(int ax, int ay, int mx, int my, BYTE color, int grav,
                       int warp, int s)
//vyrobi 1 bublinku na ax:ay (stred) s rychlosti mx:my, barvou color a grav.
{
   if (count<MAXDROP)
   {
      list[count]=new BubbleDrop(gfx,ax,ay,mx,my,color,grav, s);
      list[count]->warp = warp;
      count++;
   }
}

//==========================================================================
void DropList::Move()
//posachuje a pripadne vyhodi
{
   for (int i=0; i<count; i++)
   {
      if (list[i]->Move() == OUT_OF_BOUNDS)
      //vypadla z obrazovky! ZRUSIT!
      {
         count--;
         delete list[i];
         list[i]=list[count];
         list[count]=NULL;
         i--;
      }
   }
}

//==========================================================================
void DropList::RemoveAll()
{
   for (int i=0; i<count; i++)
      delete list[i];
   count=0;
}
