/*****************************************************************************
                                Handling maps                               
******************************************************************************
  Written by : Vojvo  & G.
  Language   : VisuC 6 + DX6.1

  2009-12-25 / mensi cistka stareho vykreslovaciho kodu
  2010-09-29 / prvni pokus o premocitavani souradnic

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "consts.h"
#include "map.h"
#include "dx8graph.h"
#include "tex8manager.h"
#include "exceptions.h"
#include "fakedir.h"

#define TILE_TRANSPARENT   32768


//==========================================================================
Map load_map (char *name)
{
   FILE *fr;
   Map dest = NULL;

   // alokace
   dest = new WORD[MAP_SIZE];

   if ((fr = fakeopen("data\\levels.dir",name,"rb")) == NULL)
   {
      if (dest != NULL)
         delete [] dest;
      dest = NULL;
   }
      
   // nacteni
   if (dest != NULL)
   {
      ZeroMemory(dest,MAP_SIZE);
      fread((void *)dest,MAP_SIZE,2,fr);
      fclose(fr);
   }
   
   return dest;
}

//-------------------------------------------------------------------------
// nahrava vetrnou mapu
//-------------------------------------------------------------------------
WindMap load_wndmap (char *name)
{
   FILE *fr;
   WindMap dest = NULL;

   // alokace
   dest = new char[WNDMAP_SIZE*2];

   if ((fr = fakeopen("data\\levels.dir",name,"rb")) == NULL)
   {
      if (dest != NULL)
         delete [] dest;
      dest = NULL;
   }
      
   // nacteni
   if (dest != NULL)
   {
      ZeroMemory(dest,WNDMAP_SIZE*2);
      fread((void *)dest,WNDMAP_SIZE,2,fr);
      fclose(fr);
   }
   
   return dest;
}

//===========================================================================
WORD map_value (int x, int y, Map map)
{
   int offs;

   offs = (int)(x/MAP_CELX) + MAP_XSIZE*(int)(y/MAP_CELY);
   if (offs > MAP_SIZE)
      return (char) MAP_ERROR;
   return *(map+offs);
}

//-------------------------------------------------------------------------
// vraci vitr na zadane pozici
//-------------------------------------------------------------------------
char wndmap_value (int x, int y, WindMap map)
{
   int offs;

   offs = WNDMAP_YSIZE*(int)(x/MAP_CELX) + (int)(y/MAP_CELY);
   if (offs > WNDMAP_SIZE)
      return  (char) MAP_ERROR;
   return *(map+offs);
}

//==========================================================================
void draw_map2(Map map, Texture cels)
{
   RECT r,t;
	RFileHeader * rh;
   BYTE cels_r;
	int celx, cely;
   int x, y, mask;
   WORD value;
   bool transparent = false;
   
   // osahme si rozmry grafiky
	rh = textureManager->GetHeader(cels);
	cels_r = rh->byCelRow;
	celx = rh->dwCelWidth;
	cely = rh->dwCelHeight;
	//MAP_CELHD se asi me zruit
	
	// jedeme po polkch
	for (int jloc = MAP_TOP-1; jloc<MAP_BOTTOM; jloc++)
		for (int iloc = -1; iloc<MAP_XSIZE;iloc++)
		{
			// souradnice rohu policka ve "starych" souradnicich
         x = iloc * MAP_CELX;
         y = jloc * MAP_CELY;
			
			// id kostky
			/*
			// tohle jaksi funguje samo od sebe - asi ukldm do mapy i kostky za okrajem
			if (jloc < MAP_TOP)
				// pseudokostka nad hornm okrajem kvli stnu
				value = map[jloc*MAP_XSIZE];
				*/
			if (iloc < 0)
				// pseudokostka za levm okrajem kvuli stnu
				value = map[jloc*MAP_XSIZE];
			else
				// normln kostka
				value = map[jloc*MAP_XSIZE+iloc];
			
         transparent = false;
			mask = 0xffffffff;
         if (value & TILE_TRANSPARENT)
         {
            value -= TILE_TRANSPARENT;
            transparent = true;
				mask = 0xa0ffffff; // bylo80ffffff
         }
         if (value)
         {
				// je tam njak kostka, take nco nakreslme
				// textura (zdroj)
            r.left = (value % cels_r) * celx;
            r.right = r.left + celx;
            r.top = (value /cels_r) * cely;
            r.bottom = r.top + cely;
				
				// stn
				t.left = x + 10;
				t.top = y + 10;
				t.right = t.left + 20; //***
				t.bottom = t.top + 20;
				// oriznuti poslednho dku //*** nen to zbyten? To si mon oet DX - a navc to bylo patn =)
				/*
				if (jloc == MAP_BOTTOM-1) 
				{
				r.bottom -= 10;
				t.bottom -= cely / 2;
				}
				*/
				t = dx8.Coord(t);
				dx8.BltSprite(&t,cels,&r,0,0x80000000);
				/*
				// korekce poslednho dku zptky
				if (jloc == MAP_BOTTOM-1) 
				r.bottom += 10;
				*/
				
				if ((iloc >= 0) && (jloc >= MAP_TOP))
				{
					// ploinky kreslime jen na ploe
					t.left = x;
					t.top = y;
					t.right = t.left + 20; //***
					t.bottom = t.top + 20;
					t = dx8.Coord(t);
					dx8.BltSprite(&t,cels,&r,0,mask);
				}
			}
      }
}

//===========================================================================
int under(int x, int y, Map map, int bound) {
   int retv=0, s;

   int offs = (int)(x/MAP_CELX) + MAP_XSIZE * (int)(y/MAP_CELY);
   int limit = (offs % MAP_XSIZE) + (MAP_YSIZE-1) * MAP_XSIZE;

   do 
   {
      s = *(map+offs);
      if (s > 0) {
         retv++;
      }
      offs += MAP_XSIZE;
   } while (offs < limit && s > 0 && retv < bound);
   return retv;
}

//===========================================================================
//#define MAP_REPORT
int countturnpos (Map map)
{
   long tiles = 0;
   long plats = 0;
   long over =  0;
   WORD lasttile = 0;
   WORD thistile = 0;
   long res = 0;
   for (int jloc = 3; jloc<(MAP_YSIZE-2); jloc++)
   {
      lasttile = 0;
      for (int iloc = 0; iloc<MAP_XSIZE; iloc++)
      {
         thistile = map[iloc+jloc*MAP_XSIZE];
         if (thistile && !map[iloc+(jloc-1)*MAP_XSIZE])
         {
             tiles++;
             if (!lasttile) plats++;
             if ((jloc<(MAP_YSIZE-2) && map[iloc+(jloc+2)*MAP_XSIZE]) ||
                (jloc<(MAP_YSIZE-3) && map[iloc+(jloc+3)*MAP_XSIZE]) )
                over++;
         }
         lasttile = thistile;
      }
   }
   res = 100*over/tiles + 100*plats/over;
#ifdef MAP_REPORT
   FILE * f;
   if ((f = fopen("maprep.txt","a+t")) != NULL)
   {
      fprintf(f,"\ntiles: %i\n",tiles);
      fprintf(f,"plats: %i\n",plats);
      fprintf(f,"over : %i\n",over);
      fprintf(f,"res  : %i\n",res);
      fclose(f);
   }
#endif

   return (int)res;
}
