//****************************************************************************
//                   Sprite handling module, animation                        
//****************************************************************************
//  Written by : Vojvo & Gunny
//  Language   : VisuC6 + DX8
/*
   changes:

   2013-04-17 / ukazatele docasnych bonusu
   2009-12-25 / oprava diakritiky u bonus
	           / predvn sprvn barvy od hre u bonus messages
				  / oprava lapae (vytvel "neviditeln bubliny")
   2008-06-20 / hri se rzn efektuj podle stavu
	2008-06-20 / DX8

   1.1.2 / opraveny vpisy o bodovch kouzel
   1.1.1  / u hrace zavedeny promenne lastseenX (pro Murrayho)
	         pokus o opravu havarie pri plivani za chuze do zdi 
// 1.0.1  - snaha o prvni plnou verzi
// 0.99.6 - uprava skoku na dvoji gravitaci
// 0.99.5 - odkomentovan repelent=0 v Ressurection
//        - prohozeni bodovych a powerup bonusu
//        - ukazatele trvani casove omezenych kouzel 
//          (nesmrtelnost, neviditelnost, krehke bubliny, mrazik)
//        - zobrazovani zmenenych statistik jinou barvou nez nezmenenych
// 0.99.4 - repelent
// 0.11.1 - Player se pri zmene smeru za chuze okamzite zastavi a obrati
//        - Player muze plivnout i kdyz je mimo okraje - clipped
// 0.10.2 / pripraveno blikani bufferu
//        - oprava situace, kdy pri plivnuti, kdyz player dobrzdoval, klouzal
//        - kouzla krehke bubliny, vymena bodu, pozic a vlastnosti
//        - drak po doskoku pri aktivni chuzi nezabrzdi (pokud nefiruje)
// 0.10.1 - pridana dalsi kouzla (mrazik, +time)
//        - rozsiren vypis statistik
//        - zvuky stereo
//        - Player::CheckState predelan na odecitani ruzneho poctu stepu
// 0.9.3 - upraven test na posouvani sprita pri non-stable dopadu 
//         na roh plosinky
//       - zprovoznena dalsi kouzla
// 0.9.2 - funguje stav playera RESURRECTED
//       - funguje stav playera INVISIBLE
// 0.9.1 - funguji nektera kouzla
// 0.8.2 - pridany zvuky
// 0.8.1 - spravna barva kapek u hrace
*/

#include <stdio.h>
#include <string.h>
#include "consts.h"
#include "exceptions.h"
#include "dx8graph.h"

#include "tex8manager.h"
#include "keyboard.h"
#include "map.h"
#include "gs_game.h"
#include "font8.h"
#include "obj_message.h"
#include "wsound.h"
#include "gonio.h"
#include "property.h"
#include "framesv.h"
#include "sprites.h"


#define AT_BOTTOM      0    // navr.hod. IsStable, kdyz je to na dne

#define MASK_INVISIBLE		0x607f7f7f //0x40c0c0c0
#define MASK_FROZEN			0x407fbfff
#define MASK_INVULNERABLE	0x60ff3f3f
#define MASK_SELFBLOWING	0x40ffe700

#define DURATION_RADIUS		28
#define DURATION_STEP		6


//#########################################################################
//--------------------------------- Sprite --------------------------------
//#########################################################################
Sprite::Sprite (int onx, int ony, Property *prop)
{
   type = COMMON;
   id = prop->id;
   fTgs = prop->fTgs;
   direction = DIR_L;
   celw = prop->rh->dwCelWidth;
   celh = prop->rh->dwCelHeight;
   celr = prop->rh->byCelRow;
	if (prop->rhb)
	{
		bcelw = prop->rhb->dwCelWidth;
		bcelh = prop->rhb->dwCelHeight;
		bcelr = prop->rhb->byCelRow;
	}
   xsize = prop->xsize;
   ysize = prop->ysize;
   b_miny = PLAYFIELD_MIN_Y - ysize;
   b_maxy = PLAYFIELD_MAX_Y - ysize;
   b_maxx = PLAYFIELD_MAX_X - xsize;
   eb_maxy = PLAYFIELD_EMAX_Y - ysize;
   xoff = prop->xoff;
   yoff = prop->yoff;
   frames.frames = &(prop->frames);
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   jumpNext();   
   gfx = prop->surf;
   bubgfx = prop->bubsurf;
   collision = 0;
   gravity = DFT_GRAVITY;
   newx = oldx = onx + xoff;
   newy = oldy = ony + yoff;
   mul_x = newx << SHIFT_MOVE + MUL_HALF;
   mul_y = newy << SHIFT_MOVE + MUL_HALF;
   movex = movey = 0;
   stable_l = prop->stablel;
   stable_r = prop->stabler;
   hitx = prop->hitx;
   hity = prop->hity;
	margin = prop->margin;
   topx = prop->speed;
   topy = MAX_FALLSPEED;
   bottomx = -topx;
   bottomy = -JUMP_STRENGTH;
   sort_priority = 0;
   is_stable = 0;
   clipped = false;
   frameLeft = 0;
   frameTop = 0;
}

//-------------------------------------------------------------------------
// animace
//-------------------------------------------------------------------------
void Sprite::jumpNext(int steps)
{
   frames.jumpNext(steps);
   if (frames.chainid == CHAIN_BUBBLE || frames.chainid == CHAIN_SCRAPE)
   {
		// *** nutn doeit! velikost sprajta a bubliny se muzou lisit!
      //frameLeft = BUB_XSIZE * frames.frameid;
		/*
      if (celw > 40) frameLeft = (frames.frameid % celr) * 128;
		else frameLeft = BUB_XSIZE * frames.frameid;
      frameTop = 0;
		//*** tady to vazne!
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + 128;  //xsize //***
      frameRect.bottom = frameTop + 128;	//ysize //***
		*/
      frameLeft = (frames.frameid % bcelr) * bcelw;
      frameTop = (frames.frameid / bcelr) * bcelh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + bcelw;  //xsize
      frameRect.bottom = frameTop + bcelh;	//ysize
   }
   else
   {
      frameLeft = (frames.frameid % celr) * celw;
      frameTop = (frames.frameid / celr) * celh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + celw;  //xsize
      frameRect.bottom = frameTop + celh;	//ysize
   }
}

//=========================================================================
void Sprite::Paint()
{
	RECT r, s;

	if (frozen.timecurrent > 0)	// puvodne xcurrent
		// kresli se ukazatel zmrzlosti
	{
		LONG centerx = newx + xsize / 2;
		LONG centery = newy + ysize / 2; 
		
		LONG fid; // frame id ukazatele
		
					 /*
					 bylo by korektn zjiovat si velikost framu z grafiky, ale te se mi to nechce eit - V.
					 vim, e polko m 128x128 a jsou 4 na dek
		*/
		LONG cols = 4;
		LONG framex = 128;
		LONG framey = 128;
		LONG radius = DURATION_RADIUS;	// tady je konstantni
		DWORD durmask;
		
			fid = 7 - frozen.xcurrent; // 0 je max, posledn je 7
			if (fid < 0) fid = 0;
			s.top = fid / 4 * framey;
			s.bottom = s.top + framey;
			s.left = (fid % 4) * framex;
			s.right = s.left + framex;
			
			r.left = centerx - radius; 
			r.right = centerx + radius;
			r.top = centery - radius;
			r.bottom = centery + radius;
			r = dx8.Coord(r);
			
			durmask = MASK_FROZEN;
			if (frozen.xcurrent <= 8) 
			{
				durmask += fid * 0x10000000;
				if (frozen.changedx()) 
					durmask |= 0x3fffffff;
			}
			dx8.BltSprite(&r, duration_gfx, &s, 0, durmask);

	}

	// ted samotny sprajt

	r.top = newy; //y
	r.left = newx;
	r.bottom = r.top + ysize; // +(frameRect.bottom-frameRect.top);
	r.right = r.left + xsize; // +(frameRect.right-frameRect.left);
	
	r = dx8.Coord(r);
	//dx8.BltSprite(&r, gfx, &frameRect, 0);
	dx8.BltMotionBlur(&r, &lastPos, gfx, &frameRect, 0);
	
	lastPos = r;
   
   oldx = newx;
   oldy = newy;


}

//=========================================================================
void Sprite::PaintShadow()
{
	// prhlednost m takhle sice o jeden frame zpodn, ale to by nemlo vadit...
	DWORD localmask = mask;
	localmask = ((localmask >> 4) * 6) & 0xff000000;
	RECT r;
	r.top = newy+5;
	r.left = newx+5;
	r.bottom = r.top + ysize; // + (frameRect.bottom-frameRect.top);
	r.right = r.left + xsize; // + (frameRect.right-frameRect.left);
	r = dx8.Coord(r);
	dx8.BltSprite(&r, gfx, &frameRect, 0, localmask);
	//dx8.BltMotionBlur(&r, &lastPos, gfx, &frameRect, 0, localmask); // asi to nestoj za to...
	
}

//========================================================================
int Sprite::Move()
{
   int result = 0;
   
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   if (newx<PLAYFIELD_MIN_X) {
      newx = PLAYFIELD_MIN_X;
      mul_x = newx << SHIFT_MOVE + MUL_HALF;
      movex = 0;
      result |= SP_COL_L;
   }
   else if (newx>b_maxx) {
      newx = b_maxx;
      mul_x = newx << SHIFT_MOVE + MUL_HALF;
      movex = 0;
      result |= SP_COL_R;
   }
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   if (newy<PLAYFIELD_EMIN_Y) {
      newy = PLAYFIELD_EMIN_Y;
      mul_y = newy << SHIFT_MOVE;
      movey = 0;
      result |= SP_COL_U;
   }
   else if (newy>eb_maxy) {
      frameRect.bottom = frameRect.top + ysize;
      newy = PLAYFIELD_EMIN_Y;
      mul_y = newy << SHIFT_MOVE;
      //    movey = 0;
      result |= SP_COL_D;
   }
   collision |= result;
   return result;
}

//==========================================================================
void Sprite::SlowIt(int by_x, int by_y)
{
   if (movex > 0)
   {
      if (movex > by_x)
         movex -= by_x;
      else
         movex = 0;
   }
   else
   {
      if (movex < (-by_x))
         movex += by_x;
      else
         movex = 0;
   }
   if (movey > 0)
   {
      if (movey > by_y)
         movey -= by_y;
      else
         movey = 0;
   }
   else
   {
      if (movey < (-by_y))
         movey += by_y;
      else
         movey = 0;
   }
}

//==========================================================================
int Sprite::Slow  (int by_x, int by_y)
{
   int result = 0;
   
   movex -= by_x;
   movey -= by_y;
   if (movex < bottomx) {
      movex = bottomx;
      result = 1;
   }
   if (movey < bottomy) {
      movey = bottomy;
      result = 1;
   }
   return result;
}
//==========================================================================
int Sprite::Haste  (int by_x, int by_y)
{
   int result = 0;
   
   movex += by_x;
   movey += by_y;
   if (movex > topx) {
      movex = topx;
      result = 1;
   }
   if (movey > topy) {
      movey = topy;
      result = 1;
   }
   return result;
}

//==========================================================================
int Sprite::IsStable()
{
   int result = 0;
   int parx,pary,top_move,y;
   
   // je stabilni ?
   if (newy >= PLAYFIELD_MAX_Y)
   {
      result = AT_BOTTOM;
   }
   else
   {
      y = newy;
      // podle smeru, x-bod, ktery musi byt stabilni
      if (direction == DIR_R)
         parx = newx+stable_r;
      else
         parx = newx+stable_l;
      
      pary = y+ysize;
      top_move = (topy >> SHIFT_MOVE);
      //
      result=map_value(parx,pary,map);
      if (abs(movey)<MUL_MOVE)
      {
         result = result && !map_value(parx,pary-2,map);
         if (result && map_value(parx,pary-1,map))
         {
            newy -= 1;
            mul_y -= MUL_MOVE;
         }
      }
      else
      {
         result = result && !map_value(parx,pary-top_move-2,map);
         if (result && map_value(parx,pary-top_move-1,map))
         {
            newy -= 1;
            mul_y -= MUL_MOVE;
         }
      }
   }
   is_stable = result;
   return result;
}
//==========================================================================
int Sprite::CantGoLR()
{
   int result,result3,parx1,parx2,pary,y;
   
   result3 = 0;
   y = newy; 
   if (is_stable)
   {
      pary = y+ysize-(topy>>SHIFT_MOVE)-1;
   }
   else
   {
      pary = y+ysize-1;
   }
   
   if (direction == DIR_R)
   {
      parx1 = newx+xsize-1-margin;	// pravy okraj celeho policka
      parx2 = newx+stable_r;	// pravy bod stability
   }
   else
   {
      parx1 = newx+margin;				// leva strana
      parx2 = newx+stable_l;
   }
   
   if (map_value(parx1,pary,map) != 0)        // nohy
   {
      if (map_value(parx2,pary,map) != 0)
      {
         result3 = 0;
      }
      else
      {
         if (map_value(parx1,pary+MAP_CELY,map) == 0)
            if (movey > 0)
               result3 = -1; // v souctu vynuluje nutne nasledujici result4 
            else
            {
               if (direction == DIR_L)
               {
                  if (map_value(parx1+MAP_CELX,pary+MAP_CELY,map) == 0)
                     result3 = -1;
                  else
                     result3 = 1;
               }
               else
               {
                  if (map_value(parx1-MAP_CELX,pary+MAP_CELY,map) == 0)
                     result3 = -1;
                  else
                     result3 = 1;
               }
            }
         else
            result3 = 1;
      }
   }
   
   // --- novy Roryho test ---
   int result4 = 0;
   int under1 = under(parx1,pary,map,8);
   int under2 = under(parx2,pary,map,under1);
   if (under1 > under2)
   {
      result4 = 1;
   }
   
#ifdef FILEDEBUG
   if (type == MONSTER)
      fprintf(fw,"%p -> (%d,%d):%d R:%d TEST:%d,%d UNDER %d,%d\n",this,parx1,
      parx2,newy+ysize-pary,(direction > 0),result3,result4,under1,under2);
#endif
   
   result = result3 + result4;
   if (result)
   {
      if (direction == DIR_R)
         collision |= SP_COL_R;
      else
         collision |= SP_COL_L;
   }
   return (result);
}



//##########################################################################
//------------------------------- Creature ---------------------------------
//##########################################################################

//===========================================================================
void Creature::MapReflex()
{
   IsStable();
   
   // osetreni "prochazeni zdi" doleva a doprava
   if (CantGoLR())
   {
		/*
      // kdyz jde doleva je treba ho posunout o jeden zpatky
		// 2011
		// ...zd se, e to nen nutn
		// dky tomu, e se obrt, jsou ty 1-2 pixely pesahu tolerovateln
		// nebo dokonce douc
		// navc stable je obvykle vy (cca 10), take by to nemlo nikde vadit

      if (direction == DIR_L)
		{
			// narazil vlevo
         newx = (newx/MAP_CELX+1) * MAP_CELX;
		}
      else
		{
			// narazil vpravo
         //newx = (newx/MAP_CELX+1) * MAP_CELX - (xsize % MAP_CELX);
			newx = (newx/MAP_CELX) * MAP_CELX - (xsize % MAP_CELX);
		}
		*/

      mul_x = (newx << SHIFT_MOVE) + MUL_HALF;
      movex = 0;
   }
   
   // osetreni dopadu na plosinku
   if ((action & SPMA_JUMP) && movey > 0 && is_stable)
   {
      sound_device->Play(2,newx+xsize/2);
      newy = (newy/MAP_CELY) * MAP_CELY + yoff;
      mul_y = newy << SHIFT_MOVE;
      movey = 0;
      action &= ~(SPMA_LEFT + SPMA_RIGHT);
      if (!(action & SPMA_FIRE))
      {
         frames.setChain(CHAIN_STAND);
//         action &= ~(SPMA_LEFT + SPMA_RIGHT);
      }
      else
      {
         movex = 0;
         old_chain = CHAIN_STAND;         
      }
      action ^= SPMA_JUMP;
   }
   
   // osetreni spadnuti na konci plosinky
   if (!(action & SPMA_JUMP) && !is_stable)
   {
      action |= SPMA_JUMP;
      old_chain = frames.chainid;
      if (!(action & SPMA_FIRE)) frames.setChain(CHAIN_FALL);
   }
}

//===========================================================================
void Creature::Reincarnate()
{
   movex = 0;
   movey = 0;
   action = 0;
}


//##########################################################################
//-------------------------------- Player ----------------------------------
//##########################################################################
Player::Player(Property *prop, DWORD col) : Creature (0,0,prop)
{
   type = PLAYER;
   dik_left = dik_right = dik_jump = dik_fire = 0;
   sc_last = 0;
	colour = col;
   state = 0;
   time_resurrected.options(15000,0,60,true,false);
   time_invisible.options(15000,0,60,true,false); 
   time_selfblowing.options(15000,0,60,true,false);
   time_resurrected.decrease(time_resurrected.timecurrent);
   time_invisible.decrease(time_invisible.timecurrent);
   time_selfblowing.decrease(time_selfblowing.timecurrent);
   updateable = 1;
   //
   jump_strength = JUMP_STRENGTH;
   bottomy = -MOVE600;
   acceleration = MOVE_ACCEL;
   bubble_speed = SPEED_BUBBLE;
   //bubble_delay = 500;
   repelent = 0;
   gravityLow = gravity;
   gravityHigh = gravity*2;
	zoom = 0;
}

//-------------------------------------------------------------------------
// usadi hrace na spravne misto do levelu a vubec ho pripravi na hru V 2004
//-------------------------------------------------------------------------
void Player::Place(int x, int y)
{
	// takze pozice v pixelech
   newx = x * MAP_CELX;
   newy = y * MAP_CELY + PLAYFIELD_TOP;

   frames.setChain(CHAIN_STAND);
	// ktery smerem se diva
   if (x>15) 
   {
      direction=DIR_L;
      frames.setDirection(DIR_L);
		// drak nema obvyklych 40pix, tak at je to presne
		newx += 40 - xsize;
   }
   else 
   {
      direction=DIR_R;
      frames.setDirection(DIR_R);
   }
   jumpNext();
   
   oldx = newx;
   mul_x = newx << SHIFT_MOVE;
   oldy = newy;
   mul_y = newy << SHIFT_MOVE;
   sheet.energy = sheet.maxenergy;
   Reincarnate();
   state &= IMMORTAL;

	// pro jistotu nejak nastavim lastseen
	// (nemelo by to mit vliv, protoze to je jen na prvni pruchod smyckou)
	lastseenx = newx;
	lastseeny = newy;

	keyhintstarttime = 0;

}


//-------------------------------------------------------------------------
// nastavuje klavesy
//-------------------------------------------------------------------------
void Player::SetKeys(BYTE dikl,BYTE dikr,BYTE dikj,
                     BYTE dikf)
{
   dik_left = dikl;
   dik_right = dikr;
   dik_jump = dikj;
   dik_fire = dikf;
	keyhint = true;
}

//-------------------------------------------------------------------------
// reaguje na aktualni stav playera
//-------------------------------------------------------------------------
void Player::stateReflex()
{
   if (!(state & INBUBBLE))
   {
      // kdyz skace, zpomaluje ho gravitace
      if (action & SPMA_JUMP)
      {
         if (movey > 0 && frames.chainid == CHAIN_JUMP)
         {
            if (action & SPMA_FIRE)
            {
               old_chain = CHAIN_FALL;
            }
            else
            {
               frames.setChain(CHAIN_FALL);
               jumpNext();
            }
         }
         Haste(0,gravity);
      }
      // ukonceni firingu; je treba nastavit ukazovatko na puvodni framy
      if ((action & SPMA_FIRE) && frames.chainid != CHAIN_FIRE)
      {
         frames.setChain(old_chain);
         action &= ~(SPMA_FIRE | SPMA_LEFT | SPMA_RIGHT);
      }
      //kdyz probiha fire neprijma se zadna akce
      if (!(action & SPMA_FIRE))
      {
         if (action & SPMA_LEFT)
         {
            if (direction == DIR_R && !(action & SPMA_JUMP))
               movex = 0;
            Slow(acceleration,0);
            if (movex < 0 && direction == DIR_R)  // otoceni spritu
            {
               direction = DIR_L;
               frames.setDirection(DIR_L);
               jumpNext();
               newx += stable_r-stable_l;
               mul_x = newx << SHIFT_MOVE + MUL_HALF;
            }
         }
         else if (action & SPMA_RIGHT)
         {
            if (direction == DIR_L && !(action & SPMA_JUMP))
               movex = 0;
            Haste (acceleration,0);
            if (movex > 0 && direction == DIR_L)  // otoceni spritu
            {
               direction = DIR_R;
               frames.setDirection(DIR_R);
               jumpNext();
               newx -= stable_r-stable_l-1;
               mul_x = newx << SHIFT_MOVE + MUL_HALF;
            }
         }
      }
   }
}

//==========================================================================
void Player::KeyReflex()
// reakce na stisk klaves:  sc_left .... doleva
//                          sc_right ... doprava
//                          sc_jump .... skok
//                          sc_fire .... vypusteni bubliny     
{
   BYTE last = 0;
   Player *otherplayer;
   
   // kdyz hrac pusti klavesu fire povol jeji opetovny stisk
   if (fk_pressed && !KEYDOWN(kbd.diKeys,dik_fire))
      fk_pressed = false;
   
   if (action & SPMA_JUMP && gravity == gravityLow && !KEYDOWN(kbd.diKeys,dik_jump))
   {
      gravity = gravityHigh;
   }
   if (!(state & INBUBBLE))
   {
		// neni v bubline
      // kdyz probiha fire, neprijma se zadna jina akce
      if (!(action & SPMA_FIRE))
      {
         // hrac drzi klavesu "vlevo"
         if (KEYDOWN(kbd.diKeys,dik_left)) 
         {
            if (action==0) 
            {
               frames.setChain(CHAIN_WALK);
               jumpNext();
            }
            action |= SPMA_LEFT;
            last |= SCL_LEFT;
         }
         else
         {
            action &= ~SPMA_LEFT;
         }
         // hrac drzi klavesu "vpravo"
         if (KEYDOWN(kbd.diKeys,dik_right)) 
         {
            if (action==0) 
            {
               frames.setChain(CHAIN_WALK);
               jumpNext();
            }
            action |= SPMA_RIGHT;
            last |= SCL_RIGHT;
         }
         else
         {
            action &= ~SPMA_RIGHT;
         }
         // hrac stiskl klavesu "skok" a pajdulak prave neskace
         if (KEYDOWN(kbd.diKeys,dik_jump) && !(action & SPMA_JUMP))
         {
            sound_device->Play(3,newx+xsize/2);
            action |= SPMA_JUMP;
            frames.setChain(CHAIN_JUMP);
            jumpNext();
            Slow (0,jump_strength);
            gravity = gravityLow;
            last |= SCL_JUMP;
         }
         // hrac stiskl klavesu "fire" a nedrzi ji uz od minuleho fire
         // a neni orezan - mimo hraci pole
			// to by mohl neco vyplivnout =)
         if (KEYDOWN(kbd.diKeys,dik_fire) && !fk_pressed)
         {
				// fajn... tak kam s bublinou?
            long x = newx;
            long y = newy;
				// vhodna pozice X
            if (direction == DIR_L)
               x = x + xsize - BUB_XSIZE;
				// V. 2004 - kontrola na kraji obrazovky
				if (x > (PLAYFIELD_MAX_X - BUB_XSIZE))
				{
					x = PLAYFIELD_MAX_X - BUB_XSIZE;
				}
				if (x < (PLAYFIELD_MIN_X))
				{
					x = PLAYFIELD_MIN_X;
				}
				// eof 2003

				// vhodna pozice Y
            if (newy < PLAYFIELD_MIN_Y && newy > 10)
            {
               y = 40;
            }
            else if (newy > (PLAYFIELD_MAX_Y - ysize) && newy < 430)
            {
               y = 400;
            }
            else if (clipped)
               y = 0;
            if (y > 0)
            {
               action |= SPMA_FIRE;
               old_chain = frames.chainid;
               if (old_chain == CHAIN_WALK)
                  old_chain = CHAIN_STAND;
               frames.setChain(CHAIN_FIRE);
               if (!(action & SPMA_JUMP)) 
               {
                  // zrusime chuzi doleva|doprava
                  movex=0;
               }
               fk_pressed = true;
					// ma jeste co plivat?
               if (!sheet.energy)
               {
                  sound_device->Play(29,newx+xsize/2); //skytaci sampl
                  drops->OneDrop(newx+((direction)?34:6),y+8,0,-MOVE25,dropcolor, -1,10, DROP_PAINT-random(2)); // V. 2006 - puvodne gravitace 0 a -MOVE50
               }
               else
               { 
						// jo, rezervni bubliny by byly
                  int bubid = ID_BUBBLE;
                  if (this->id == ID_PLAYER1)
                      bubid = ID_SOLIDBUBBLEG;
                   else
                      bubid = ID_SOLIDBUBBLEB;
                  Bubble *pom = new Bubble(x,y,properties.member[bubid]);
                  pom->frames.setChain(CHAIN_RELEASE);
                  pom->jumpNext();
                  // uprava na bonus 'krehke bubliny'
                  if (time_selfblowing.timecurrent > 0)
                  {
                     pom->selfblowing = 1;
                  }               
                  sound_device->Play(0,newx+xsize/2);
                  pom->whose = this;
                  pom->dropcolor = dropcolor;
                  sheet.energy--;
                  if (ARRAY_FULL(qbubbles_ctrl))
                  {
                     delete pom;
                  }
                  else
                  {
                     ARRAY_ADD(qbubbles,qbubbles_ctrl,pom)
                        if (direction == DIR_R)
                           pom->movex = bubble_speed;
                        else
                           pom->movex = -bubble_speed;
                  }
               }            
               last |= SCL_FIRE;
            }
         }
      }
      // hrac nedrzi zadnou klavesu
      if (!last)
      {
         // kdyz neni skok nebo fire tak zrus chuzi
         if (!(action & SPMA_JUMP))
         {
            if (!(action & SPMA_FIRE)) 
            {
               movex = 0;
               action = 0;
               if (sc_last)
                  frames.setChain(CHAIN_STAND);
            }
            else
            {
               action &= ~(SPMA_LEFT + SPMA_RIGHT);
               old_chain = CHAIN_STAND;
            }
         }
         else
            action &= ~(SPMA_LEFT + SPMA_RIGHT);
      }
      else
      {
         // zhmotneni
         if (state & NOTTAKEN)
         {
            sound_device->Play(12,newx+xsize/2);
            state ^= UNCONTROLLED;
            // nastaveni nezranitelnosti
            state |= RESURRECTED;
            time_resurrected.increase(1500);
            otherplayer = (this == player1) ? player2 : player1;
            --player1->sheet.lives;
				player1->sheet.UpdateView();

				// pokud je to poprve ve hre nebo je druhy hrac v tomhle levelu jeste v puvodnim stavu, tak u neho zacit fadovat
				if (keyhint || (otherplayer->keyhintstarttime == 0)) 
					otherplayer->keyhintstarttime = (time&(0xffffffff-2047)); // aby se vzdycky dokoncil fadeout
				keyhint = false;

				//player1->sheet.AddLives(0); //*** zatm hack, aby se udlal refresh
            if ((otherplayer->state & NOTTAKEN) && !player1->sheet.lives)
            {
               // Don't let reincarnate otherplayer 
               otherplayer->state |= DEAD;
               //ARRAY_REMOVE(monsters,monsters_ctrl,otherplayer);
            }
         }
      }
   }
   else
   {
		// je v bubline
      if (KEYDOWN(kbd.diKeys,dik_fire) && !fk_pressed)
      {
         state |= WANNALEAVE;
         fk_pressed = true;
			// vrzanim prozradi Murraymu svoji pozici
			lastseenx = newx;
			lastseeny = newy;
      }
   }
   sc_last = last;
}

//==========================================================================
int Player::CollideWith(Sprite *what)
{
   int result, value;
   int diff_x,diff_y;
   
   result = 0;
   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   switch (what->type)
   {
   case BONUS: if (diff_x < 0 && -diff_x < (what->xsize-PLAYERBONUS_HITTOLX) || 
                   diff_x >= 0 && diff_x < (xsize-PLAYERBONUS_HITTOLX))
                  if (diff_y < 0 && -diff_y < (what->ysize-PLAYERBONUS_HITTOLY) || 
                      diff_y >= 0 && diff_y < (ysize-PLAYERBONUS_HITTOLY))
                  {
                     result = 1;
                     value = ((Bonus *)what)->value;
                     sheet.AddPoints(value,what->newx + what->xsize/2,what->newy+10);
                     ((Bonus *)what)->counter = BONUS_LIFE;
                  }
                  break;
   case MONSTER: if (diff_x < 0 && -diff_x < (what->xsize-what->hitx) || 
                     diff_x >= 0 && diff_x < (xsize-what->hitx))
                    if (diff_y < 0 && -diff_y < (what->ysize-what->hity) || 
                        diff_y >= 0 && diff_y < (ysize-PLAYERMONSTER_HITTOLY))
                    {
                       result = 1;
                       Die();
                    }
                    break;
   }
   return result;
}


//==========================================================================

void Player::Die() 
{
   state &= ~(INVISIBLE);
   frames.setChain(CHAIN_DYING);
   sound_device->Play(19,newx+xsize/2);
   state = state | DEAD;
   movex = 0;
   movey = 0;
   sheet.koef = 1;
   UFO *ufo;
   for (int i=0; i<UFOcnt; i++)
   {
      ufo = UFOs[i];
      if (ufo->id==ID_MURRAYB || ufo->id==ID_MURRAYG)
         if (((Murray *)ufo)->target==this)
            ufo->Melt();
   }

	if (s_drops) 
	{
        drops->Blast(newx + xsize/2,newy + ysize*3/4-1,DROP_RED,gravitation);
        drops->Blast(newx + xsize/2,newy + ysize*3/4+1,DROP_WHITE,gravitation+4);
	}

}

//==========================================================================
void Player::Resurrection()
{
   state |= UNCONTROLLED;
   state &= ~(INACTIVE);
   
   time_resurrected.set(0);
   time_invisible.set(0);
   time_selfblowing.set(0);
   //repelent = 0; //zatim ho necham jako jednu z vlastnosti
   
   frames.setChain(CHAIN_STAND);
   Reincarnate();
	zoom = 0;
}

//==========================================================================
void Player::Paint()
{
   RECT r, s;
   int y = newy;
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
	// je normalne na plose

	// vykreslen ukazatel s trvnm bonus
	if (!(state & NOTTAKEN)) // ma smysl jen kdyz je ve hre
	{
		LONG centerx = newx + xsize / 2;
		LONG centery = newy + ysize / 2; 
		
		LONG fid; // frame id ukazatele
		
		/*
		bylo by korektn zjiovat si velikost framu z grafiky, ale te se mi to nechce eit - V.
		vim, e polko m 128x128 a jsou 4 na dek
		*/
		LONG cols = 4;
		LONG framex = 128;
		LONG framey = 128;
		LONG radius = DURATION_RADIUS;	// postupne se muze zvetsovat
		LONG rstep = DURATION_STEP;		// taky se postupne zvetsuje
		DWORD durmask;

		// nezranitelnost
		
		if (time_resurrected.timecurrent > 0) //xcurrent
		{
			fid = 7 - time_resurrected.xcurrent; // 0 je max, posledn je 7
			if (fid < 0) fid = 0;
			s.top = fid / 4 * framey;
			s.bottom = s.top + framey;
			s.left = (fid % 4) * framex;
			s.right = s.left + framex;
			
			r.left = centerx - radius; 
			r.right = centerx + radius;
			r.top = centery - radius;
			r.bottom = centery + radius;
			r = dx8.Coord(r);
			
			durmask = MASK_INVULNERABLE;
			if (time_resurrected.xcurrent <= 8)
			{
				durmask += fid * 0x10000000;
				if (time_resurrected.changedx()) 
					durmask |= 0x1fffffff;
			}
			dx8.BltSprite(&r, duration_gfx, &s, 0, durmask);

			radius += rstep;
			rstep++;
		}

		// neviditelnost
		
		if (time_invisible.timecurrent > 0)
		{
			fid = 7 - time_invisible.xcurrent; // 0 je max, posledn je 7
			if (fid < 0) fid = 0;
			s.top = fid / 4 * framey;
			s.bottom = s.top + framey;
			s.left = (fid % 4) * framex;
			s.right = s.left + framex;
			
			r.left = centerx - radius; 
			r.right = centerx + radius;
			r.top = centery - radius;
			r.bottom = centery + radius;
			r = dx8.Coord(r);
			
			durmask = MASK_INVISIBLE;
			if (time_invisible.xcurrent <= 8)
			{
				durmask += fid * 0x10000000;
				if (time_invisible.changedx()) 
					durmask |= 0x1fffffff;
			}
			dx8.BltSprite(&r, duration_gfx, &s, 0, durmask);
			
			radius += rstep;
			rstep++;
		}

		// krehke bubliny
		if (time_selfblowing.timecurrent > 0)
		{
			fid = 7 - time_selfblowing.xcurrent; // 0 je max, posledn je 7
			if (fid < 0) fid = 0;
			s.top = fid / 4 * framey;
			s.bottom = s.top + framey;
			s.left = (fid % 4) * framex;
			s.right = s.left + framex;
			
			r.left = centerx - radius; 
			r.right = centerx + radius;
			r.top = centery - radius;
			r.bottom = centery + radius;
			r = dx8.Coord(r);
			
			durmask = MASK_SELFBLOWING;
			if (time_selfblowing.xcurrent <= 8)
			{
				durmask += fid * 0x10000000;
				if (time_selfblowing.changedx()) 
					durmask |= 0x3fffffff;
			}
			dx8.BltSprite(&r, duration_gfx, &s, 0, durmask);
			
			radius += rstep;
			rstep++;
		}
	}

	// tak to vykresleni
   if (!clipped || frameRect.bottom > frameRect.top)  
   {
      // pasivni hrac
      if ((state & NOTTAKEN))
      {
			if (player1->sheet.lives > 0)
			{
				// pokud je k dispozici, tak blika
				int tt = (time&1023);
				if (time&1024) tt = 1024 - tt;
				mask = (tt<<21)|0x01ffffff;
				// jmno hre
				fonts[1]->Print(dx8.CoordX(newx+18),dx8.CoordY(y-10),sheet.name,colour & mask,ALIGN_CENTER, 255, dx8.tilesize / 2);

				// za urcitych podminek zobrazuje klvesov hint
				if (s_keyhint && keyhint)
				{
					if ((keyhintstarttime ==0) || (time - keyhintstarttime < 2048)) // 2048 je jeden fade
					{
						// src
						s.top = 0;
						s.left = 0;
						s.right = 64;	//keys_gfx->
						s.bottom = 64;
						
						// dst
						long grid = dx8.tilesize * 3 / 10;
						long startx = dx8.CoordX(newx+18) - grid * 9 / 2;
						long starty = dx8.CoordY(y) - 6 * grid;

						
						// aby byla cela cedule na obrazovce
						if (startx < 0)
							startx = 0;
						if (startx + 9 * grid >= dx8.sizex)
							startx = dx8.sizex - 9 * grid;
						
						char wrk[4] = " ";
						DWORD doublemask = (mask<<1)|1;

						r.left = startx;
						r.top = starty;
						r.right = r.left + 2*grid;
						r.bottom = r.top + 2*grid;
						dx8.BltSprite(&r, keys_gfx,&s,0, doublemask);
						wrk[0] = AsKey[dik_fire];
   					fonts[0]->Print(r.left + grid, r.top-1, wrk,colour & doublemask,ALIGN_CENTER, 255, 2*grid);

						r.left += 3*grid;
						r.right += 3*grid;
						r.top += 2*grid;
						r.bottom += 2*grid;
						dx8.BltSprite(&r, keys_gfx,&s,0, doublemask);
						wrk[0] = AsKey[dik_left];
   					fonts[0]->Print(r.left + grid, r.top-1, wrk,colour & doublemask,ALIGN_CENTER, 255, 2*grid);

						r.left += 2*grid;
						r.right += 2*grid;
						r.top -= 2*grid;
						r.bottom -= 2*grid;
						dx8.BltSprite(&r, keys_gfx,&s,0, doublemask);
						wrk[0] = AsKey[dik_jump];
   					fonts[0]->Print(r.left + grid, r.top-1, wrk,colour & doublemask,ALIGN_CENTER, 255, 2*grid);
						
						r.left += 2*grid;
						r.right += 2*grid;
						r.top += 2*grid;
						r.bottom += 2*grid;
						dx8.BltSprite(&r, keys_gfx,&s,0, doublemask);
						wrk[0] = AsKey[dik_right];
   					fonts[0]->Print(r.left + grid, r.top-1, wrk,colour & doublemask,ALIGN_CENTER, 255, 2*grid);

					}
				}
			}
			else
			{
				// pokud dosly zivoty, tak by mel zmizet
				mask = 0;
			}
      }
      // neviditelny
      else if (state & INVISIBLE)
      {
			mask = 0x60606060;
      }
      // nezranitelny
      else if ((state & RESURRECTED) && (time_resurrected.timecurrent & 64)) //64
      {
			mask = 0xffff7f7f;
      }
		// normalni
		else
		{
			mask = 0xffffffff;
		}

		// kreslime draka
		r.top = newy; //y
		r.left = newx;
		r.bottom = r.top + (ysize); // +(frameRect.bottom-frameRect.top);
		r.right = r.left + (xsize); // +(frameRect.right-frameRect.left);

		// rozplvn mrtvoly
		if (zoom)
		{	
			//r.top -= zoom*;
			//r.bottom += zoom*2;
			//r.left -= zoom*6;
			//r.right += zoom*6;

			DWORD localalpha = 255 - zoom/2;
			if (localalpha > 255) 
				localalpha = 0;
			if (localalpha < 0) localalpha = 0;
			localalpha = (localalpha << 24 | 0x00ffffff);
			mask = mask & (localalpha);
		}

		r = dx8.Coord(r);
		
		//dx8.BltSprite(&r, gfx, &frameRect, 0, mask);
		dx8.BltMotionBlur(&r, &lastPos, gfx, &frameRect, 0, mask);
		lastPos = r;

		// pokud je ve he, kreslime bublinovy buffer
		if (! (state & (NOTTAKEN | DEAD)))
		{
			r.left = r.right;
			if (direction) r.left -= (dx8.CoordX(xsize) + DROP_PAINT);
			r.right = r.left + DROP_PAINT;
			r.bottom = r.top + DROP_PAINT;

			//
			RECT energy0cel_rect = {DROP_TILE*5,0,DROP_TILE*6,DROP_TILE};
			RECT energy1cel_rect = {DROP_TILE*(id-1),0,DROP_TILE*id,DROP_TILE};
			int i = 0;
			while (i < sheet.maxenergy)
			{
				if (i < sheet.energy)
					dx8.BltSprite(&r,drops->gfx,&energy1cel_rect,0, mask);
				else 
					dx8.BltSprite(&r,drops->gfx,&energy0cel_rect,0,0x80ffffff & mask);
				r.top += DROP_PAINT+1;
				r.bottom += DROP_PAINT+1;
				i++;
			}
		}

   }
}

//==========================================================================
void Player::BubbleAction(int bubx, int buby)
{
   newx = bubx;
   newy = buby;
}

//==========================================================================
void Player::CheckState(int stepcnt)
{
   if (! (state & NOCREDIT))
   {
      // nezranitelnost
      if (state & RESURRECTED)
      {
         if (time_resurrected.decrease(stepcnt) == 0)
            state ^= RESURRECTED;
      }
      // neviditelnost
      if (state & INVISIBLE)
      {
         if (time_invisible.decrease(stepcnt) == 0)
            state ^= INVISIBLE;
      }
      // samopraskaci bubliny
      if (time_selfblowing.timecurrent > 0)
      {
         time_selfblowing.decrease(stepcnt);
      }

   }
	
	// automaticka aktualizace posledni viditelne pozice, pokud je viditelny
	if (!(state & (INVISIBLE | INBUBBLE)))
	{
		lastseenx = newx;
		lastseeny = newy;
	}
}


//##########################################################################
//---------------------------------- Monster -------------------------------
//##########################################################################
Monster::Monster(int onx, int ony, Property *prop) : Creature (onx,ony,prop)
{
   decision = 1 + random(1);
   type = MONSTER;
   kill_points = prop->value1;
   catch_points = prop->value2;
   bonus_cr = 0;
}

//-------------------------------------------------------------------------
// obnoveni vlastnosti pri vyadnuti z bubliny
//-------------------------------------------------------------------------
void Monster::Reincarnate()
{
   movex = 0;
   movey = 0;
   decision = decision = 1 + random(1);
   action = 0;
}

//-------------------------------------------------------------------------
// test na stabilitu
//-------------------------------------------------------------------------
int Monster::IsStable()
{
   int result,parx,pary,top_move;
   
   if (newy >= PLAYFIELD_MAX_Y) //pri testech by chmatal mimo mapu
   {
      result = AT_BOTTOM;
   }
   else
   {
      
      // podle smeru, x-bod, ktery musi byt stabilni
      if (direction == DIR_R)
         parx = newx+stable_r;
      else
         parx = newx+stable_l;
      
      pary = newy + ysize;
      top_move = topy >> SHIFT_MOVE;
      
      // je stabilni ?
      result=map_value(parx,pary,map);
      if (abs(movey)<MUL_MOVE)
      {
         result = result && !map_value(parx,pary-2,map);
         if (result && map_value(parx,pary-1,map))
         {
            newy -= 1;
            mul_y -= MUL_MOVE;
         }
      }
      else
      {
         result = result && !map_value(parx,pary-top_move-2,map);
         if (result && map_value(parx,pary-top_move-1,map))
         {
            newy -= 1;
            mul_y -= MUL_MOVE;
         }
      }
   }
   wasstable = is_stable = result;
   return result;
}

//==========================================================================
void Monster::KeyReflex()
{
   char last;
   int oldmovex;
   
   last = 0;
   if (action & SPMA_JUMP)
   {
      if (movey>0 && frames.chainid != CHAIN_FALL) 
         frames.setChain(CHAIN_FALL);
      Haste(0,gravity);
   }
   
   //rozhodovani o smeru pohybu
   //obrat pri narazu
   if (collision & (SP_COL_L|SP_COL_R))
   {
      decision^=3;
      action^=3; //tohle prihazuju navic... kvuli kralika
      collision=0;
      // je to jedno... stejne se o collision nikdo jinde nezajima
   }
   //pohyb podle rozhodnuti
   if ((decision&SCL_LEFT)==SCL_LEFT) {
      if (!action) 
         frames.setChain(CHAIN_WALK);
      action |= SPMA_LEFT;
      oldmovex = movex;
      Slow(MOVE_ACCEL,0);
      if (oldmovex >= 0 && movex < 0) 
      {
         direction = DIR_L;
         frames.setDirection(DIR_L);
      }
      last |= SCL_LEFT;
   }
   if ((decision&SCL_RIGHT)==SCL_RIGHT) {
      if (!action) 
         frames.setChain(CHAIN_WALK);
      action |= SPMA_RIGHT;
      oldmovex = movex;
      Haste (MOVE_ACCEL,0);
      if (oldmovex <= 0 && movex > 0)
      {
         direction = DIR_R;
         frames.setDirection(DIR_R);
      }
      last |= SCL_RIGHT;
   }
   if ((decision & SPMA_JUMP) && !(action & SPMA_JUMP)) {
      action |= SPMA_JUMP;
      frames.setChain(CHAIN_JUMP);
      Slow(0,JUMP_STRENGTH);
      last |= SCL_JUMP;
      decision ^= SCL_JUMP;
   }
   if (!last) {
      if (!((action & SPMA_FIRE) || (action & SPMA_JUMP)))
         frames.setChain(CHAIN_STAND);
   }
}

//==========================================================================
void Monster::BubbleAction(int bubx, int buby)
{
   newx = bubx;
   newy = buby;
}

//##########################################################################
//---------------------------------- Z o m b -------------------------------
//##########################################################################
void Zomb::KeyReflex ()
{
   char last;
   BYTE oldindex;
   
   last = 0;
   if (action & SPMA_JUMP) 
   {
      Haste(0,gravity);
   }
   
   //rozhodovani o smeru pohybu
   //a obrat pri narazu
   if (collision & 3) 
   {
      //if (!(action & SPMA_JUMP)) // 2011: tohle dlalo fastzombovi problm v blew1
         wannajump++;
      decision^=3;
      collision=0;
      //   action&=(!3); //to je hodne divne
   }
   //pohyb podle rozhodnuti doleva
   if ((decision&SCL_LEFT)==SCL_LEFT)
   {
      Slow(MOVE_ACCEL,0);
      if (movex < 0) 
         direction = DIR_L;
      if (frames.direction == DIR_R)
      {
         oldindex = frames.index;
         frames.setDirection(DIR_L);
         frames.index = 16-oldindex;
      }
      action &= ~(SPMA_RIGHT);
      action |= SPMA_LEFT;
      last |= SCL_LEFT;
   }
   //pohyb podle rozhodnuti doprava
   if ((decision&SCL_RIGHT)==SCL_RIGHT)
   {
      Haste (MOVE_ACCEL,0);
      if (movex > 0) 
         direction = DIR_R;
      if (frames.direction == DIR_L)
      {
         oldindex = frames.index;
         frames.setDirection(DIR_R);
         frames.index = 16-oldindex;  
      }
      action &= ~(SPMA_LEFT);
      action |= SPMA_RIGHT;
      last |= SCL_RIGHT;
   }
   // poskoci!
   if (wannajump>6)
		// 2011 bylo 6, ale ignorovalo skakani
		// ovem 8 zase nkde moc dlouho trvalo
	{
      if (!(action & SPMA_JUMP)) 
		{
         wannajump=0;
         action |= SPMA_JUMP;
         Slow (0,JUMP_STRENGTH);
      }
      last |= SCL_JUMP;
   }
}

//===========================================================================
void Zomb::MapReflex()
{
   IsStable();
   
   // osetreni "prochazeni zdi" doleva a doprava
   if (CantGoLR())
   {
      newx = (newx/MAP_CELX) * MAP_CELX;
      // kdyz jde doleva je treba ho posunout o jeden zpatky
      if (direction == DIR_L)
      {
         newx += MAP_CELX;
      }
      if (abs(movex) > MUL_MOVE) 
         sound_device->Play(6,newx+xsize/2);
      mul_x = (newx << SHIFT_MOVE) + MUL_HALF; //jednodussi verze
      movex = 0;
   }
   // osetreni dopadu na plosinku
   if ((action & SPMA_JUMP) && (movey > 0) && is_stable)
   {
      if (movey > 100) 
         sound_device->Play(6,newx+xsize/2);
      newy = (newy/MAP_CELY) * MAP_CELY;
      mul_y = newy << SHIFT_MOVE;
      movey = (487-movey);      //tlumeny odraz
      movey = ((movey*3)/5);
      if (movey>=-196)
      {
         movey = 0;
         action ^= SPMA_JUMP;
      }
   }
   // osetreni spadnuti na konci plosinky
   if (!(action & SPMA_JUMP) && !is_stable)
   {
      action |= SPMA_JUMP;
   }
}


//##########################################################################
//---------------------------------- Nebulus -------------------------------
//##########################################################################
void Nebulus :: KeyReflex ()
{
   char last;
   
   last = 0;
   if (action & SPMA_JUMP)
   {
      if ((movey > 0) & (frames.chainid != CHAIN_FALL)) 
         frames.setChain(CHAIN_FALL);
      Haste(0,gravity);
   }
   
   //obrat pri narazu
   if (collision&3)
   {
      decision^=3;
      collision &= (~3);
   }
   //pohyb podle rozhodnuti
   if ((decision&SCL_LEFT)==SCL_LEFT) {
      action |= SPMA_LEFT;
      Slow(MOVE_ACCEL,0);
      if (direction == DIR_R && movex < 0) 
      {
         direction = DIR_L;
         frames.setDirection(DIR_L);
      }
      last |= SCL_LEFT;
   }
   if ((decision&SCL_RIGHT)==SCL_RIGHT) {
      action |= SPMA_RIGHT;
      Haste (MOVE_ACCEL,0);
      if (direction == DIR_L && movex > 0) 
      {
         direction = DIR_R;
         frames.setDirection(DIR_R);
      }
      last |= SCL_RIGHT;
   }
   if (!(action & SPMA_JUMP)) {
      action |= SPMA_JUMP;
      frames.setChain(CHAIN_JUMP);
      //znahodneni sily odrazu
      Slow (0,2*JUMP_STRENGTH/3+random(JUMP_STRENGTH));
      last |= SCL_JUMP;
   }
}

//##########################################################################
//---------------------------------- Motylek -------------------------------
//##########################################################################
Butterfly::Butterfly(int onx, int ony, Property *prop):Monster(onx,ony,prop)
{
   direction = DIR_L;
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   int angle = random(360);
   movex=sinx(angle);
   movey=cosx(angle);
   sort_priority = 1;
}

//========================================================================
int Butterfly::Move()
{
   int result = 0;
   
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   if (newx<PLAYFIELD_MIN_X) {
      newx = PLAYFIELD_MIN_X;
      mul_x = newx << SHIFT_MOVE;
      movex=-movex;
      sound_device->Play(7,newx+xsize/2);
      result |= SP_COL_L;
   }
   else if (newx>b_maxx) {
      newx = b_maxx;
      mul_x = newx << SHIFT_MOVE;
      movex=-movex;
      sound_device->Play(7,newx+xsize/2);
      result |= SP_COL_R;
   }
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   if (newy<PLAYFIELD_MIN_Y) {
      newy = PLAYFIELD_MIN_Y;
      mul_y = newy << SHIFT_MOVE;
      movey=-movey;
      sound_device->Play(7,newx+xsize/2);
      result |= SP_COL_U;
   }
   else if (newy>b_maxy) {
      newy = b_maxy;
      mul_y = newy << SHIFT_MOVE;
      movey=-movey;
      sound_device->Play(7,newx+xsize/2);
      result |= SP_COL_D;
   }
   collision |= result;
   return 0;
}

//===========================================================================
void Butterfly::MapReflex()
{
}

//==========================================================================
void Butterfly::KeyReflex ()
{
}

//===========================================================================
void Butterfly::Reincarnate()
{
   int angle = random(360);
   movex = sinx(angle) ;
   movey = cosx(angle);
}


//##########################################################################
//---------------------------- Druhy motylek -------------------------------
//##########################################################################
void Butterfly2::MapReflex()
{
   //tady se bude resit odrazeni od plosinek
   int mapx = newx/MAP_CELX + 1;
   int mapy = (newy/MAP_CELY + 1) * MAP_XSIZE;
   BYTE res = 0;
   //zjistim, jestli se dostal na jine policko (a nebyl uz predtim ve zdi)
   // (to by melo byt ochranou proti vytuhnuti nekde ve zdi)
   if ((mapx!=oldmapx || mapy!=oldmapy) && map[oldmapx+oldmapy] == 0)
   {
      //co na nem je?
      WORD tile = map[mapx + mapy];
      WORD tilex = map[oldmapx + mapy];
      WORD tiley = map[mapx + oldmapy];
      if (tile)
      {
         //zmenilo se jen y
         if (oldmapx == mapx) 
            res = 2;
         else
            //zmenilo se jen x
            if (oldmapy == mapy) 
               res = 1;
            else
            {
               //presel krizem
               // bud jsou vedle jine plosinky
               if (tilex) 
                  res += 2;
               if (tiley) 
                  res += 1;
               // nebo je to jedina plosinka okolo
               else
                  if (!tilex) 
                     res = 3;
            }
      }
      else 
         if (tilex && tiley && oldmapx!=mapx && oldmapy!=mapy)
            res=3;
   }
   //pokud je duvod, zmenim smer pohybu a posunu, kde byl
   if (res)
   {
      sound_device->Play(7,newx+xsize/2);
      newx = oldx;
      newy = oldy;
      mul_x = mul_prevx;
      mul_y = mul_prevy;
      if (res & 1) 
         movex=-movex;
      if (res & 2) 
         movey=-movey;
   }
   else
   {
      mul_prevx = mul_x;
      mul_prevy = mul_y;
      oldmapx = mapx;
      oldmapy = mapy;
   }
}

//==========================================================================
void Butterfly2::Reincarnate()
{
   int angle = random(360);
   movex = sinx(angle);
   movey = cosx(angle);
   
   oldmapx = newx/MAP_CELX + 1;
   oldmapy = (newy/MAP_CELY + 1) * MAP_XSIZE;
   mul_prevx = mul_x;
   mul_prevy = mul_y;
}


//##########################################################################
//-------------------------- Horizontalni motyl ----------------------------
//##########################################################################
Butterfly3::Butterfly3(int onx, int ony, Property *prop)
:Butterfly(onx,ony,prop)
{
   if (random(1))
      movex = MOVE100;
   else 
      movex= -MOVE100;
   movey=0;
}

//==========================================================================

void Butterfly3::Reincarnate()
{
   if (random(1))
      movex = MOVE100;
   else 
      movex= -MOVE100;
   movey=0;
}

//==========================================================================
void Butterfly3::MapReflex()
{
   //tady se bude resit odrazeni od plosinek
   int mapx = newx/MAP_CELX + 1;
   int mapy = (newy/MAP_CELY + 1) * MAP_XSIZE;
   //je na novem policku do ceho narazit
   if (map[mapx + mapy])
   {
      char oldmapx=oldx/MAP_CELX + 1;
      if (!map[oldmapx + mapy])
      {
         sound_device->Play(7,newx+xsize/2);
         movex=-movex;
      }
   }
}


//#########################################################################
//--------------------------------- SymBuf --------------------------------
//#########################################################################
void SymBuf::Add(Symbol s)
{
   type <<= 8;
   type += s + 1;   
}

//=========================================================================
void SymBuf::Realize(Player *p)
{
   char * mess = NULL;
   Creature * m;
   Bubble * b;
   int i;
   int timeaddon;

   if (!(player1->state & NOTTAKEN) && !(player2->state & NOTTAKEN))
      timeaddon = 15000;
   else
      timeaddon = 7500;

   //bonusy pro jednoho
   switch (type)
   {
      // docasne
   case 0x01020304 :
      p->state |= RESURRECTED;
      p->time_resurrected.increase(15000);
      mess = "Nezranitelnost";
      break;
   case 0x02020303 :
      p->state |= INVISIBLE;
      p->time_invisible.increase(15000);
      mess = "Neviditelnost";
      break;
   case 0x02020201 :
      p->time_selfblowing.increase(15000);
      mess = "Kehk bubliny";
      break;
   case 0x01010104 :
      frozen.increase(7500);
      mess = "Mrazk";
      break;
      // vlastnosti
   case 0x01010102 :
      if (p->jump_strength < MOVE650) //at nepretece
      {
         p->jump_strength += MOVE25;
         mess = "Vy vskok";
      }
      break;
   case 0x02020203 :
      if (p->acceleration<45) //at nepretece
      {
         p->acceleration += 5;
         mess = "Zven hbitosti";
      }
      break;
   case 0x03030304 :
      if (p->topx<MOVE350) //at nepretece
      {
         p->topx += MOVE25;
         p->bottomx = -p->topx;
         mess = "Vy rychlost";
      }
      break;
   case 0x04040401 :
      if (p->bubble_speed<MOVE750) //at nepretece
      {
         p->bubble_speed += MOVE50;
         mess = "Rychlej bubliny";
      }
      break;
   case 0x01020303 :
      if (p->repelent<300)
      {
         p->repelent += 55;
         mess = "Repelent";
      }
      break;
   case 0x01010202 :
      if (p->sheet.maxenergy < MAX_ENERGY) //at nepretece
      {
         p->sheet.maxenergy++;
      }
      if (p->sheet.energy<p->sheet.maxenergy)
      {
         p->sheet.energy++;
         mess = "Bublina navc";
      }
      break;
   case 0x03030404 :
      if (p->sheet.koef<10)
      {
         p->sheet.koef ++;
         mess = "Vt vivnost";
      }
      break;
      // bodove
   case 0x01010101 :
      p->sheet.AddPoints(250);
      mess = "+250";
      break;
   case 0x02020202 :
      p->sheet.AddPoints(500);
      mess = "+500";
      break;
   case 0x03030303 :
      p->sheet.AddPoints(1000);
      mess = "+1000";
      break;
   case 0x04040404 :
      p->sheet.AddPoints(2000);
      mess = "+2000";
      break;
      // specialni
   case 0x04030201 :
      if (player1->sheet.AddLives())
      {
         mess = "ivot navc";
      }
      break;
   case 0x01020202 :
      if (monstercnt > 0) // aby se as nezvyoval v redalertu
      {
         if (!(player1->state & NOTTAKEN) && !(player2->state & NOTTAKEN))
            timer.Add(15);
         else
            timer.Add(8); //*** HE?!
         mess = "30 sekund navc";
      }
      break;
   case 0x01010103 :
      for (i=0; i<monsters_ctrl.count; i++)
      {
         m = ARRAY_AT(monsters,i);
         if (m->type==MONSTER && !(ARRAY_FULL(qbubbles_ctrl)))
         {
				int bubid = ID_BUBBLE;
				if (p->id == ID_PLAYER1)
					bubid = ID_SOLIDBUBBLEG;
				else
					bubid = ID_SOLIDBUBBLEB;
            b = new Bubble(m->newx,m->newy,properties.member[bubid]);
            b->whose = p;
            b->dropcolor = p->dropcolor;
            b->counter = BUB_START-1;
            if (m->newx > 300)
               b->movex = MOVE100;
            else
               b->movex = -MOVE100;
            ARRAY_ADD(qbubbles,qbubbles_ctrl,b)
         }
		}
      mess = "Lapa";
      break;
   case 0x01010301 :
      for (i=0; i<sbubbles_ctrl.count; i++)
      {
         b = ARRAY_AT(sbubbles,i);
         if (b->counter > BUB_DOMINATED)
         {
            b->counter = BUB_DOMINATED + i*50;
            b->sender = p;
         }
      }
      mess = "Praska";
      break;
   }
    
   //bonusy, ktere maji vyznam jen pro dva aktivni hrace
   if (mess==NULL)
   { //pokud uz je nejaka hlaska, nejaky bonus uz se delal, takze nic
      if (!(player1->state & NOTTAKEN) && !(player2->state & NOTTAKEN))
      {
         long swap;
         int sw;
         BYTE sc;
         switch (type)
         {
         case 0x01030103 :
            swap = player1->sheet.score;
            player1->sheet.score = player2->sheet.score;
            player2->sheet.score = swap;
            mess = "Vmna bod";
            break;
         case 0x01020102 :
            sw = player1->mul_x;
            player1->mul_x = player2->mul_x;
            player2->mul_x = sw;
            sw = player1->mul_y;
            player1->mul_y = player2->mul_y;
            player2->mul_y = sw;
            mess = "Vmna pozic";
            break;
         case 0x01040104 :
            // zrychleni (hbitost)
            sw = player1->acceleration;
            player1->acceleration = player2->acceleration;
            player2->acceleration = sw;
            // vyskok
            sw = player1->jump_strength;
            player1->jump_strength = player2->jump_strength;
            player2->jump_strength = sw;
            // repelent
            sw = player1->repelent;
            player1->repelent = player2->repelent;
            player2->repelent = sw;
            // rychlost bublin
            sw = player1->bubble_speed;
            player1->bubble_speed = player2->bubble_speed;
            player2->bubble_speed = sw;
            // bodovy koeficient
            sw = player1->sheet.koef;
            player1->sheet.koef = player2->sheet.koef;
            player2->sheet.koef = sw;
            // maximalni rychlost
            sw = player1->topx;
            player1->topx = player2->topx;
            player2->topx = sw;
            player1->bottomx = - (player1->topx);
            player2->bottomx = - (player2->topx);
            //
            mess = "Vmna vlastnost";
            break;
         case 0x02030203 :
            sc = player1->dik_left;
            player1->dik_left = player2->dik_left;
            player2->dik_left = sc;
            sc = player1->dik_right;
            player1->dik_right = player2->dik_right;
            player2->dik_right = sc;
            sc = player1->dik_jump;
            player1->dik_jump = player2->dik_jump;
            player2->dik_jump = sc;
            sc = player1->dik_fire;
            player1->dik_fire = player2->dik_fire;
            player2->dik_fire = sc;
            mess = "Vmna hr";
            break;
         default : return;
         }
      }
      else
      { //nepadlo to na zadny bonus...
         return;
      }
   }
   //a zarizeni vypisu hlasky
   sound_device->Play(23);
   message.Options(320,190);
   message.Set(mess, p->colour);
   flash = 255; // 31 << 3 + zbytek
}
      
//=========================================================================
void SymBuf::Paint()
{
   BYTE *b;
   b = (BYTE*) &type;
   RECT r;

   int i = 0;
   while ((b[i] != 0) && (i < 4) && (b[i] < 5))
   {
		if (dx8.wide)
		{
			r.top = y;
			r.bottom = r.top + size;
			r.left = x + (3-i) * size;
			r.right = r.left + size;
			/*
			r.top = dx8.sizey - (dx8.tilesize *3 ) / 4;
			r.bottom = r.top + dx8.tilesize / 2;
			r.left = dx8.CoordX(x)+(3-i)* dx8.tilesize / 2;
			r.right = r.left + dx8.tilesize / 2;
			*/
		}
		else
		{
			// kvli zaokrouhlovn je pesnj pout fixn posun
			// je teba pepotvat a vslednou souadnici
			r.top = y;
			r.left = x+(3-i)*20;
			r.bottom = r.top + 20;
			r.right = r.left + 20;
			r = dx8.Coord(r);
		}

		dx8.BltSprite(&r, sym_bubgfx, &(symrect[b[i]-1]),0); // bublinov symboly byly v tiles, ale te jsou extra

   // blikani po slozeni bonusu
   if (flash)
   {
		int flashsize = (256-flash)/3;

		r.top -= flashsize / 2;
		r.bottom += flashsize / 2;
		r.left -= (i-1) * flashsize;
		r.right -= (i-2) * flashsize;

		dx8.BltSprite(&r, sym_bubgfx, &(symrect[b[i]-1]),0,0x00ffffff|(flash<<24)); // bublinov symboly byly v tiles, ale te jsou extra
   }
      
		i++;
   }
}

//=========================================================================
void SymBuf::UpdateFlash()
{
  if (flash) 
  {
    flash--;
  }
}



