/**************************************************************************
                               Various utils                               
***************************************************************************
   Written by : Vojvo & Gunny
   Language   : VisuC 6 
***************************************************************************
   changes:

    7. 3.2002 / pridana funkce powerof2
    X. X.2001 / ponekud ocesano pro pouziti v Balleru
   30. 5.2000 - pridana funkce line2words

**************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "utils.h"

char pom_string[256];


//-------------------------------------------------------------------------
// rozdeli radek textu na slova (oddelovace jsou mezery, 
// tabelatory a konce radku)
//-------------------------------------------------------------------------
char **line2words(const char *line)
{
   char **res_data;
   char cur_word[128];
   int res_size = 10;
   int wordcnt = 0;
   int charcnt = 0;
   int i = 0;
   char c;

   res_data = (char **) malloc(res_size*4);
   res_data[0] = NULL;

   while (line[i] != '\0')
   {
      c = line[i];
      if (c == ' ' || c == '\t' || c == '\n')
      {
         if (charcnt > 0)
         {
            if (wordcnt == (res_size-1))
            {
               res_size += 10;
               res_data = (char **) realloc(res_data,res_size*4);
            }
            res_data[wordcnt++] = strdup(cur_word);
            res_data[wordcnt] = NULL;
            charcnt = 0;
         }
      }
      else if (charcnt < 127)
      {
         cur_word[charcnt++] = c;
         cur_word[charcnt] = '\0';
      }
      i++;
   }
   if (charcnt > 0)
   {
      if (wordcnt == (res_size-1))
      {
         res_size += 10;
         res_data = (char **) realloc(res_data,res_size*4);
      }
      res_data[wordcnt++] = strdup(cur_word);
      res_data[wordcnt] = NULL;
      charcnt = 0;
   }
   return res_data;
}

//-------------------------------------------------------------------------
// prevadi cisla na text
//-------------------------------------------------------------------------
void numtos(long numb, int width, char *target)
{
   target[width] = 0;
   for(int i=width; i>0; i--)
   {
      target[i-1] = (numb % 10) + '0';
      numb /= 10;
   }
}

//-------------------------------------------------------------------------
// vraci nejblizsi vetsi mocninu dvou
//-------------------------------------------------------------------------
unsigned long powerof2(unsigned long numb)
{
   if (numb<=64)
   {
      if (numb<=16)
      {
         if (numb<=8)
            return 8;
         else
            return 16;
      }
      else
      {
         if (numb<=32)
            return 32;
         else
            return 64;
      }
   }
   else
   {
      if (numb<=256)
      {
         if (numb<=128)
            return 128;
         else
            return 256;
      }
      else
      {
         if (numb<=512)
            return 512;
         else
            return 1024;
      }
   }
}

//-------------------------------------------------------------------------
//------------------------------ struct Timer -----------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Timer::Timer()
{
   options(1000,0,100,true,true);
}

//-------------------------------------------------------------------------
// konstruktor s parametry
//-------------------------------------------------------------------------
Timer::Timer(int timemax,unsigned long xmin,unsigned long xmax,
             bool cutmax,bool ltor)
{
   options(timemax,xmin,xmax,cutmax,ltor);
}

//-------------------------------------------------------------------------
// aktualizuje xcurrent - tanhle blob se neustale opakuje, tak at je extra
//-------------------------------------------------------------------------
void Timer::updatex()
{
	lastxcurrent = xcurrent;
	if (ltor)
	{
		xcurrent = xmax - timecurrent/timeunit;
	}
	else
	{
		xcurrent = xmin + timecurrent/timeunit;
	}

	// zmenila se delka ukazatele
	if (lastxcurrent != xcurrent)
	{
		lastxchanged = timecurrent;	// ponekud nepresne - melo by to rekonstruovat presnou delku
	}
}

//-------------------------------------------------------------------------
// nastavuje vlastnosti timeru
//-------------------------------------------------------------------------
void Timer::options(int timemax,unsigned long xmin,unsigned long xmax,
                    bool cutmax,bool ltor)
{
   this->timemax = timemax;
   this->xmin = xmin;
   this->xmax = xmax;
   this->cutmax = cutmax;
   this->ltor = ltor;
   
   xcurrent = ltor ? xmin : xmax;
	lastxcurrent = xcurrent;
   timeunit = timemax / (xmax - xmin);
   timecurrent = timemax;
	lastxchanged = -1;
}


//-------------------------------------------------------------------------
// nastavuje casovac na zadanou hodnotu
//-------------------------------------------------------------------------
void Timer::set(int value)
{
   timecurrent = value;
   if (timecurrent < 0)
      timecurrent = 0;
   if (timecurrent > timemax)
   {
      if (!cutmax)
      {
         timemax = timecurrent;
         timeunit = timemax / (xmax-xmin);
      }
      timecurrent = timemax;
      xcurrent = ltor ? xmin : xmax;
   }
   else
   {
		updatex();
   }
}

//-------------------------------------------------------------------------
// snizuje timer o zadany pocet milisekund
//-------------------------------------------------------------------------
int Timer::decrease(int amount)
{
   timecurrent -= amount;
   if (timecurrent < 0)
      timecurrent = 0;

	updatex();
   return timecurrent;
}

//-------------------------------------------------------------------------
// zvysuje timer o zadany pocet milisekund
//-------------------------------------------------------------------------
int Timer::increase(int amount)
{
   timecurrent += amount;
   if (timecurrent > timemax)
   {
      if (!cutmax)
      {
         timemax = timecurrent;
         timeunit = timemax / (xmax-xmin);
      }
      timecurrent = timemax;
      xcurrent = ltor ? xmin : xmax;
   }
   else
   {
		updatex();
   }
   return xcurrent;
}

//-------------------------------------------------------------------------
// asi 200 ms po pixelov zmn ukazatele vraci true
//-------------------------------------------------------------------------
bool Timer::changedx()
{
	bool res = false;
	if (abs(timecurrent - lastxchanged) < 15) res = true;
	return res;
}
