/*
Windows DXSound
v 1.0.1
Gunny & Vojvo of Eriador
14.04.2000

  part of Bubwiny
*/

/*
1.0.1    plna verze
0.99.3	reverse_stereo
0.99  	getvolume
0.98 	pridan vypocet panningu podle zadane x-souradnice
0.97  	setvolume
0.96  	opravena chyba u SBlive
0.95	prvni fungujici verze

  ? Pokud pri cteni samplu ze souboru neco neklapne (delka),
  mel bych zmenit i parametry samplu ve strukturach directsoundu?
  Vypada to logicky...
*/

#include <stdio.h>
#include "utils.h"
#include "wsound.h"

#define MAX_PANNING     9500


WSound *sound_device = NULL;


//===========================================================================
PolySample::PolySample()
// proste vynuluje vnitrek
{
   active = 0;
   copies = 0;
   for (int l=0; l<MAXPOLY; l++)
   {
      buffer[l] = NULL;
   }
}

//===========================================================================
PolySample::DeleteSample()
// proste vynuluje vnitrek
{
   for(int i=0; i < MAXPOLY; i++)
   {
      if (buffer[i] != NULL)
      {
         buffer[i]->Release();
         buffer[i] = NULL;
      }
   }
}


//===========================================================================
LPDIRECTSOUNDBUFFER PolySample::Active()
{
   LPDIRECTSOUNDBUFFER res = buffer[active];
   active++;
   if (active>=copies) active = 0;
   return res;
} // Active

//===========================================================================
WSound::WSound(HWND w, GUID *pguid, char * filenam) 
// alokuje zarizeni, pamet a naloaduje samply
{
   // inicializuju si svoje vnitrni promenne
   werror = 0;
   winhandle = w;
   strcpy(filename,filenam);
   strcpy(errorname,"OK");
   max_x = 640;
   reverse_stereo = false;
   
   primarybuffer = NULL;
   dsound = NULL;
   
   // alokuju DirectSound
   if FAILED(DirectSoundCreate(pguid, &dsound, NULL))
   {
      werror = WERR_NODEVICE;
      strcpy(errorname,"no device available");
      return;
   }
   
   // nastavim pracovni rezim 
   if FAILED(IDirectSound_SetCooperativeLevel(dsound, winhandle, DSSCL_PRIORITY)) //PRIORITY
   {
      werror = 2;
      strcpy(errorname,"can't set cooperative");
      return;
   }
   
   // aloknu primarni buffer na michani
   ZeroMemory(&bufferdescriptor, sizeof(DSBUFFERDESC));
   bufferdescriptor.dwSize = sizeof(DSBUFFERDESC);
   bufferdescriptor.dwFlags = DSBCAPS_PRIMARYBUFFER | DSBCAPS_CTRLVOLUME;
   if FAILED(dsound->CreateSoundBuffer(&bufferdescriptor, &primarybuffer, NULL))
   {
      werror = WERR_CANTALLOCATE;
      strcpy(errorname,"can't create primary buffer");
      return;
   }
   
   // budu potrebovat nejaka data - zjistim, jaky mam soubor
   if (werror = OpenFile())
   {
      // soubor mimo dosah? =(
      strcpy(errorname,"bad file");
      // tak at se tam aspon neco tvari
      filehead.count = 0;
      filehead.frequency = 12500;
      return;
   }
   
   // uz znam format, tak muzu nastavit format zvuku
   result = SetPrimaryBuffer(2);  //zatim dame stereo
   if (result != DS_OK)
   {
      DEBUG_MESSAGE("WSound::setprimarybuffer failed.\n");
   }
   
   // alokuju sekundarni buffery na samply
   if (werror = AllocateSamples())
   {
      strcpy(errorname,"can't allocate samples");
   }
   
   // nactu veci ze souboru
   if (werror = LoadSamples())
   {
      strcpy(errorname,"00 can't load samples");
      errorname[1] += werror;
   }
   
   // vyrobime kopie samplu
   DuplicateSamples(); // muze vracet chybu, ale neni to kriticke
   
   // a jedem! (mixer na tvrdo)
   Run(); 
} // WSound


//===========================================================================
WSound::~WSound(void)
{
   DEBUG_MESSAGE("WSound::TheEnd starting =).\n");
   if (primarybuffer)
   {
      primarybuffer->Release();
      primarybuffer = NULL;
   }
   for(int i=0; i<MAXSAMPLES; i++)
      sample[i].DeleteSample();
   if (dsound)
   {
      dsound->Release();
      dsound = NULL;
   }
   DEBUG_MESSAGE("WSound::TheEnd ending.\n");
} // ~WSound

//===========================================================================
int WSound::OpenFile()
{
   int i;
   DEBUG_MESSAGE("WSound::OpenFile\n");
   // je treba si kryt zada - vsechno pro jistotu vynuluju
   ZeroMemory(&filehead, sizeof(NFXHeader));
   // tak ted to zkusime
   FILE * f;
   f = fopen(filename,"rb");
   if (f==NULL)
      return WERR_NOFILE;
   
   // soubor existuje
   int res;
   res = fread(&(filehead.id),6,1,f);
   if (!res)
   {
      // EOF!
      fclose(f);
      return WERR_BADFILE;
   }
   res = fread(&(filehead.frequency),2,1,f);
   if (!res)
   {
      // EOF!
      fclose(f);
      return WERR_BADFILE;
   }
   res = fread(&(filehead.count),2,1,f);
   if (!res)
   {
      // EOF!
      fclose(f);
      return WERR_BADFILE;
   }
   
   if (strcmp(filehead.id,"NoFX!"))
   {
      // Bad header!!!
      fclose(f);
      return WERR_BADFILE;
   }
   
   if (filehead.count>MAXSAMPLES)
   {
      // jenom to orezu (asi by se hodil warning)
      filehead.count = MAXSAMPLES;
   }
   
   // hlavicka sedi - ted jsou na rade hlavicky samplu
   for (i=0; i<filehead.count; i++)
   {
      res = fread(&(samplehead[i]),18,1,f); 
      //musi byt natvrdo 18
      //NE sizeof(SampleHeader).. ta je 16 nebo20! zaokrouhlovani?
      
      if (!res)
      {
         // EOF!
         fclose(f);
         return WERR_BADFILE;
      }
   }
   
   // to bylo vsechno
   fclose(f);
   DEBUG_MESSAGE("WSound::OpenFile done.\n");
   return 0;
} // OpenFile

//===========================================================================
int WSound::AllocateSamples()
{
   int res = 0;
   // musim si pripravit strukturu
   
   WAVEFORMATEX wfx;
   memset(&wfx, 0, sizeof(WAVEFORMATEX)); 
   wfx.wFormatTag = WAVE_FORMAT_PCM; 
   wfx.nChannels = 1; 
   wfx.nSamplesPerSec = filehead.frequency; 
   wfx.wBitsPerSample = 8; 
   wfx.nBlockAlign = wfx.wBitsPerSample / 8 * wfx.nChannels;
   wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
   
   // a ted postupne pres vsechny buffery
   for (int i=0; i<filehead.count; i++)
   {
      // Set up DSBUFFERDESC structure. 
      ZeroMemory(&bufferdescriptor, sizeof(DSBUFFERDESC)); 
      bufferdescriptor.dwSize = sizeof(DSBUFFERDESC);
      // DSBCAPS_CTRLDEFAULT = DSBCAPS_CTRLPAN, DSBCAPS_CTRLVOLUME, and DSBCAPS_CTRLFREQUENCY 
      // Need default controls (pan, volume, frequency). 
      bufferdescriptor.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_STATIC;
      /*
      DSBCAPS_GETCURRENTPOSITION2   // Always a good idea
      | DSBCAPS_GLOBALFOCUS         // Allows background playing
      | DSBCAPS_CTRLPOSITIONNOTIFY;
      */
      bufferdescriptor.dwBufferBytes = samplehead[i].length;
      //bufferdescriptor.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
      bufferdescriptor.lpwfxFormat = &wfx;
      // Create buffer. 
      result = (dsound->CreateSoundBuffer(&bufferdescriptor, &(sample[i].buffer[0]), NULL)); 
      //&(samplebuffer[i]) //sbuff
      if (result!=DS_OK)
      { 
         sample[i].buffer[0] = NULL;
         res |= WERR_CANTALLOCATE;
      } 
      else
      {
         // nastavim si, ze je sampl ok
         sample[i].copies = 1;
         //(samplebuffer[i])->SetVolume(DSBVOLUME_MAX); 
      }
   }
   return res;
} // AllocateSamples

//===========================================================================
int WSound::LoadSamples()
{
   DEBUG_MESSAGE("WSound::LoadSamples\n");
   int res = 0;
   for (int i=0; i<filehead.count; i++)
   {
      res |= LoadSample(i);
   }
   return res;
} // LoadSamples

//===========================================================================
int WSound::LoadSample(int num)
{
   int res = 0;
   // je vubec v souboru tolik samplu?
   if (num>=filehead.count) return WERR_NOSUCHSAMPLE;
   // je tam skutecne sampl ulozeny?
   if (samplehead[num].length==0) return 1;
   // mam alokovany buffer?
   if (sample[num].copies == 0) return WERR_CANTALLOCATE;
   //if (sbuff==NULL) return WERR_CANTALLOCATE;
   
   // tak zkusime nacist
   FILE * f;
   f = fopen(filename,"rb");
   if (f==NULL)
      return WERR_NOFILE;
   
   // skocim na spravne misto do souboru
   if (fseek(f,samplehead[num].startpos,SEEK_SET) == 0)
   {
      
      // zablokuju buffer a nactu sampl
      LPVOID p,q;
      DWORD cnt1,cnt2;  
      DWORD flags = DSBLOCK_ENTIREBUFFER;
      
      int len = samplehead[num].length;
      result = (sample[num].buffer[0])->Lock(0,len,&p,&cnt1,&q,&cnt2,flags);
      if SUCCEEDED(result)
      {
         // klaplo to - nactu sampl
         // NUTNO resit, kdyz len != cnt1 (zatim predpokladam ==)
         int rr = fread(p,1,cnt1,f);
         if (cnt2 > 0)
         {
            rr += fread(q,1,cnt2,f);
         }
         if (rr<len)
         {
            // zrejme to neni cele - nutno upravit nactenou hlavicku
            samplehead[num].length = rr;
            res = WERR_BADFILE; //co kdyz nacetl neco? chybu nehazet?
         }
         //
         result = (sample[num].buffer[0])->Unlock(p,cnt1,q,cnt2);
         if FAILED(result)
         {
            DEBUG_MESSAGE("WSound::Unlock failed.\n");
         }
      }      
   }
   else
   {
      // seek nevysel - soubor je moc kratky - EOF!
      // priste uz to nema cenu cist
      samplehead[num].length = 0;
      res = WERR_BADFILE; 
   }
   
   // vsechno
   fclose(f);
   return res;
} // LoadSample

//===========================================================================
int WSound::DuplicateSamples()
// duplikuje odpovidajici pocet kopii
{
   DEBUG_MESSAGE("WSound::DuplicateSamples\n");
   int res = 0;
   int max;
   for (int i=0; i<filehead.count; i++)
   {
      max = samplehead[i].copies;
      if (max > MAXPOLY) max = MAXPOLY;
      for (int l=1; l<max; l++)
      {
         if SUCCEEDED(dsound->DuplicateSoundBuffer(sample[i].buffer[0], &(sample[i].buffer[l])))
         {
            sample[i].copies = l+1;
         }
         else
         {
            res |= WERR_CANTDUPLICATE;
         }
      }
   }
   return res;
   
} // DuplicateSamples

//===========================================================================
int WSound::FindSample(char * samplename)
{
   int res = -1;
   for (int i=0; (i<filehead.count) && (res == -1); i++)
   {
      if (!strcmp(filename,samplehead[i].name))
         res = i;
   }
   return res;
} // FindSample

//===========================================================================
int  WSound::SetPrimaryBuffer(int channels)
{
   WAVEFORMATEX wfx;
   ZeroMemory(&wfx, sizeof(WAVEFORMATEX)); 
   wfx.wFormatTag = WAVE_FORMAT_PCM; 
   wfx.nChannels = channels; 
   wfx.nSamplesPerSec = filehead.frequency; 
   wfx.wBitsPerSample = 8; 
   wfx.nBlockAlign = wfx.wBitsPerSample / 8 * wfx.nChannels;
   wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
   
   return (primarybuffer->SetFormat(&wfx));
} // SetPrimaryBuffer

//===========================================================================
void WSound::Run()
{
   // musi byt loooped!
   primarybuffer->Play(0,0,DSBPLAY_LOOPING); // reserved, reserved, flags 
} // Run

//===========================================================================
void WSound::Play(int num, DWORD x)
{
   // pokud neklapnul init, nema to cenu
   if (werror) return;   
   // mam tolik samplu?
   if (num>=filehead.count) return;
   // mam sampl pripraveny?
   //if (samplebuffer[num]==NULL) return;
   
   LPDIRECTSOUNDBUFFER active;
   active = sample[num].Active();
   // potrebuju zjistit, jestli je buffer ok a ready
   DWORD stat; 
   result = active->GetStatus(&stat);
   if (result == DS_OK)
   {
      if (stat & DSBSTATUS_BUFFERLOST)
      {
         LoadSample(num);
      }
      if (stat & DSBSTATUS_PLAYING)
      {
         DEBUG_MESSAGE("WSound::Play already playing.\n");
         //(samplebuffer[num])->Stop();
         // kdyz tak spis active->Stop();
         active->SetCurrentPosition(0);
      }
   }
   // panning
   LONG panning;
   panning = (2*MAX_PANNING*x)/max_x - MAX_PANNING;
   if (reverse_stereo) panning = - panning;
   result = active->SetPan(panning);
   if (result != DS_OK)
   {
      DEBUG_MESSAGE("WSound::Play can't set panning.\n");
   }
   
   // uz ho jenom spustit... (nebo mozna uz nekdy nemusim?)
   if ((stat & DSBSTATUS_PLAYING) == 0)
   {
      result = active->Play(0,0,0);
      if (result != DS_OK)
      {
         DEBUG_MESSAGE("WSound::Play failed.\n");
      }
   }
} // Play int

//===========================================================================
void WSound::Play(char * samplename, DWORD x)
{
   int n = FindSample(samplename);
   if (n>=0)
      Play(n,x);
} // Play char

//===========================================================================
void WSound::SetVolume(LONG vol)
{
   primarybuffer->SetVolume(vol);
} // SetVolume

//===========================================================================
void WSound::GetVolume(LPLONG vol)
{
   primarybuffer->GetVolume(vol);
} // GetVolume


//EOF========================================================================
