/*
*--------------------------------------------------------------------
*
* Application
*
* Aplikace - nejvyssi uroven prubehu hry
*
* autori  Vojvo & G.
*
*--------------------------------------------------------------------
* zmeny:

* 0.10.2 - stav klavesnice se necte v onTimer (timer se asi vyhodi)
* 0.11.2 - zabity onTimer a timer jako takovy
* 0.9.4 - toppers jiz nejsou gamestage => ubyly herni stavy 9-11
* 0.9.8 / pridano osetrovani switche darken
* 0.99.1 - demoverze
* 0.99.2 - oprava nacitani hlasitosti z registru
* 0.99.3 - oprava spatneho nacitani z neexistujicich hodnot registru
*        - z registru se cte reverse stereo
* 0.99.4 - uprava pro AVIPlayer

 2008-06-20 / lehce zmaten reorganizace pro DX8 a bubw15
 2009-12-24 / opraveno delete fonts[2]
 2009-12-25 / v ramci zprehledneni zlikvidovani toppers
 2013-03-28	/ do registru pridano changepass, keyhint a catchy
  
*--------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <windowsx.h>
#include <vfw.h>

#include "version.h"
#include "types8.h"

//#include "mmgr.h"

#include "dx8graph.h"
#include "keyboard.h"
#include "tex8manager.h"
#include "exceptions.h"
#include "rfile32.h"
#include "font8.h"
#include "registry.h"
#include "resource.h"
#include "topper.h"
#include "utils.h"
#include "application.h"
#include "wsound.h"


Application *app;

const char * Application::szAppName = "Bubliny HD for Windows 32";
const char * Application::szWCName = "Bubliny HD";

#define DEFPASS	"XFTG9NEEB"
#define DEFNAME1	"Zelen"
#define DEFNAME2	"Modr"

//-------------------------------------------------------------------------
// implicitni konstruktor
//------------------------------------------------------------------------- 
Application::Application()
{
   m_bDDActive = false;
   m_bInitialized = false;
   m_hWnd = 0;
   game = NULL;
   menu = NULL;
   strcpy(m_szDataDir,"data\\");
   strcpy(levelname,"");
   state = 0;
   stage = NULL;
   destroyed = false;
   vol = 0;
}

//-------------------------------------------------------------------------
// cteni hodnot z registru
//-------------------------------------------------------------------------
void Application::readRegistry()
{
   DWORD dwVol = 255;
   Registry *reg;
   strcpy(s_password,DEFPASS);
   *((DWORD *)s_key[0]) = 0x02031210;
   *((DWORD *)s_key[1]) = 0x9dc8cdcb;
   strcpy(s_name[0],DEFNAME1);
   strcpy(s_name[1],DEFNAME2);
	s_catchy = false;
	s_changepass = false;
   s_drops = true;
   s_darken = 4;
   s_test = false;
   s_testsymbol = false;
	s_keyhint = true;
   s_testholy = false;
   s_reverse_stereo = false;
   s_vol = 255;
   levelname[0] = '\x0';

	reg = new Registry();
   reg->open("Bubliny HD",KEY_READ);
   if (reg->opened)
   {
      reg->readString("Heslo",s_password,12);
      if (s_password[0] == '\x0')
         strcpy(s_password,DEFPASS);
      
      reg->readString("Jmeno 1",s_name[0],32);
      if (s_name[0][0] == '\x0')
         strcpy(s_name[0],DEFNAME1);
      s_name[0][11] = '\x0';   // omezeni na 11 znaku
      
      reg->readString("Jmeno 2",s_name[1],32);
      if (s_name[1][0] == '\x0')
         strcpy(s_name[1],DEFNAME2);
      s_name[1][11] = '\x0';   // omezeni na 11 znaku
      
      pom_string[0] = '\x0';
      reg->readString("Catchy",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_catchy = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("ChangePass",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_changepass = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Drops",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_drops = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("KeyHint",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ne"))
            s_keyhint = false;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Test",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_test = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("RevStereo",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_reverse_stereo = true;
      }
      
      reg->readDWORD("Hlasitost",&dwVol);
      if (dwVol <= 10)
         s_vol = (BYTE)dwVol;

      reg->readDWORD("Darken",&dwVol);
      if (dwVol <= 10)
         s_darken = (BYTE)dwVol;
      
      reg->readDWORD("Ovladani 1",(DWORD *)s_key[0]);
      reg->readDWORD("Ovladani 2",(DWORD *)s_key[1]);
      
      reg->close();
   }
	delete reg;
}

//-------------------------------------------------------------------------
// zapis hodnot do registru
//-------------------------------------------------------------------------
void Application::writeRegistry()
{
   Registry *reg = new Registry();
   reg->create("Bubliny HD");
   reg->writeString("Heslo",s_password);
   reg->writeString("Jmeno 1",s_name[0]);
   reg->writeString("Jmeno 2",s_name[1]);
   reg->writeString("Catchy",s_catchy ? "Ano":"Ne");
   reg->writeString("ChangePass",s_changepass ? "Ano":"Ne");
   reg->writeString("Drops",s_drops ? "Ano":"Ne");
   reg->writeDWORD("Darken",(DWORD)s_darken);
   reg->writeString("Test",s_test ? "Ano":"Ne");
   reg->writeString("KeyHint",s_keyhint ? "Ano":"Ne");
   reg->writeString("RevStereo",s_reverse_stereo ? "Ano":"Ne");
   if (s_vol != 255)
      reg->writeDWORD("Hlasitost",(DWORD)s_vol);
   reg->writeDWORD("Ovladani 1",*((DWORD *)s_key[0]));
   reg->writeDWORD("Ovladani 2",*((DWORD *)s_key[1]));
   reg->close();
   delete reg;
}

//-------------------------------------------------------------------------
// inicializace aplikace
//-------------------------------------------------------------------------
bool Application::initInstance(HINSTANCE hInstance, LPSTR lpCmdLine, int nCmdShow, 
                               WNDPROC lpWndProc)
{   
   WNDCLASSEX wcex;
   
   // Set up and register window class
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.style = CS_HREDRAW | CS_VREDRAW;
   wcex.lpfnWndProc = lpWndProc;
   wcex.cbClsExtra = 0;
   wcex.cbWndExtra = 0;
   wcex.hInstance = hInstance;
   wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBFULL1));
   wcex.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBFULL1));
   wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
   wcex.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);;
   wcex.lpszMenuName = szAppName;
   wcex.lpszClassName = szWCName;
   RegisterClassEx(&wcex);
   
   MCIWndRegisterClass();
   
   SystemParametersInfo(SPI_GETWORKAREA,0,&m_rcDispArea,0);
   m_rcDispArea.right = GetSystemMetrics(SM_CXSCREEN);
   m_rcDispArea.bottom = GetSystemMetrics(SM_CYSCREEN);
   m_hInstance = hInstance;

	
#define RES3
#ifdef RES1
	DWORD winsizex = 1440;
	DWORD winsizey = 900;
#endif
#ifdef RES2
	DWORD winsizex = 1600;
	DWORD winsizey = 1200;
#endif
#ifdef RES3
	DWORD winsizex = 1024;
	DWORD winsizey = 768;
#endif


	// - PRIKAZOVA RADKA -
   // pobezi v okne?
   bool wnd = false;
   DWORD dwExStyle = WS_EX_TOPMOST;
   DWORD dwStyle = WS_POPUP;
   if (strchr(lpCmdLine,'w') != NULL) 
   {
      wnd = true;
      dwExStyle = 0;
      dwStyle = WS_OVERLAPPED | WS_SYSMENU;
		// parametry odladn na WinXP
      m_rcDispArea.right = winsizex + 6;
		m_rcDispArea.bottom = winsizey + 30;
	}

   // Create a window
   m_hWnd = CreateWindowEx(dwExStyle,
      szWCName,
      szAppName,
      dwStyle,
      0,
      0,
      m_rcDispArea.right,
      m_rcDispArea.bottom,
      NULL,
      NULL,
      hInstance,
      NULL);
   
   if (!m_hWnd)
      return false;
   
   ShowWindow(m_hWnd, nCmdShow);
   UpdateWindow(m_hWnd);
   SetFocus(m_hWnd);
   
   try
   {

     // tady bych mel vybrat, jake rozliseni pouziju
      //*** melo by se nacitat z registru!(ale mozna uz o kus driv)
      // zatim nechm autodetekci
      DWORD xsize = 0;
      DWORD ysize = 0;
      D3DFORMAT fmt = D3DFMT_UNKNOWN;
      // pokud je v okn, tak natvrdo 640,480, (D3DFMT_X8R8G8B8)
      if (wnd == TRUE)
      {
		// x:1680	y:1050	f:60	bpp:22 X8R8G8B8
		/*
         xsize = 1680; //640
         ysize = 1050; //480
			*/
         xsize = winsizex;
         ysize = winsizey;
         fmt = D3DFMT_X8R8G8B8;
      }

      // Initialize Direct3D
      if (SUCCEEDED( dx8.Init(m_hWnd, xsize, ysize, fmt,0,wnd) ) ) //wnd
      { 
         dx8.texturemanager->force16bit = 0;
			textureManager->SetDirectory(m_szDataDir);
         
			kbd.init(hInstance,app->getMainWindow());
			kbd.acquire(); 

         // fonty se pouzivaji furt, takze sem s nimi
         fonts[0] = new Font8;
         fonts[0]->Load("calibri.r24"); //b2font
         fonts[1] = new Font8;
         fonts[1]->Load("futura36.r24"); // qfontbwv
			fonts[1]->default_spacing = 0;
         fonts[2] = new Font8;
         //fonts[2]->Load("thinscore36.r24");
			fonts[2]->Load("futura.r24");
			fonts[2]->default_spacing = 0;

//--- init start

   textureManager->Load(&tiles_gfx,"tiles.r24",glSpritesFake);
   textureManager->Load(&keys_gfx,"keys.r24",glSpritesFake);
   textureManager->Load(&sym_bubgfx,"hd-bubsymbols.r24",glSpritesFake);
   textureManager->Load(&timer10_gfx,"redtimer.r24",glMiscFake); //timer10.r16
   textureManager->Load(&timealert_gfx,"hurry4.r24",glMiscFake);
   textureManager->Load(&duration_gfx,"duration.r24",glMiscFake);
   textureManager->Load(&hourglass_gfx,"hourglass.r24",glMiscFake);

   
	properties.load("data\\sprites.def"); // tohle si nacte i textury spritu
	
	sound_device = new WSound(app->getMainWindow(),NULL,"data\\BLASTER.NFX");
	if (sound_device->werror == 0)         
	{
		sound_device->GetVolume(&vol);
		if (s_vol == 255)
		{
			if (vol < -2000)
				s_vol = 0;
			else
				s_vol = BYTE((vol+2000)/200);
			if (s_vol > 10)
				s_vol = 10;
		}
		else
		{
			if (s_vol == 0)
				sound_device->SetVolume(-10000);
			else
				sound_device->SetVolume(((int)s_vol)*200-2000);
		}
	}

	toppers = new Toppers();
	if (toppers->Load())
		toppers->New();
	
//--- init end

			// game stages
			game = new Game();
			menu = new Menu();
			
			menu->load();

			stateChange();
		}
   }
   catch (Exception *exc)
   {
      DEBUG_MESSAGE("E1:");
      destroyInstance();
      MessageBox(m_hWnd,exc->msg,"Error",MB_OK);
      PostMessage(m_hWnd,WM_CLOSE,0,0);
   }
   catch (...)
   {
      DEBUG_MESSAGE("E2:");
      destroyInstance();
      MessageBox(m_hWnd,"unknown error","Error",MB_OK);
      PostMessage(m_hWnd,WM_CLOSE,0,0);
   }
   return true;
}

//-------------------------------------------------------------------------
// cinnost aplikace v dobe, kdy nechodi zadne zpravy
//-------------------------------------------------------------------------
void Application::update()
{
   bool isLost;
   
   if (m_bDDActive)
   {
      try 
      {
         isLost = true;
         try 
         {
            if (stage != NULL)
               stage->update();
            isLost = false;
         }
         catch(IsLostException *)
         {
            DEBUG_MESSAGE("update: surface lost!");
            textureManager->RestoreAll();
            stage->restore();
            //ddRestored = false;
         }
      }
      catch (Exception *exc)
      {         
         DEBUG_MESSAGE("E3:");
         destroyInstance();
         MessageBox(m_hWnd,exc->msg,"Error",MB_OK);
         PostMessage(m_hWnd,WM_CLOSE,0,0);
      }
      catch (...)
      {
         DEBUG_MESSAGE("E4:");
         destroyInstance();
         MessageBox(m_hWnd,"unknown error","Error",MB_OK);
         PostMessage(m_hWnd,WM_CLOSE,0,0);
      }
   }
   else
   {
      WaitMessage();
   }
}

//-------------------------------------------------------------------------
// uklidi aplikaci pred ukoncenim
//-------------------------------------------------------------------------
void Application::destroyInstance()
{
   if (destroyed)
      return;
   DEBUG_MESSAGE("Destroying...\n");

   kbd.done(); 
   unload();
   //if (lpDD != NULL)
     // doneDD();
   DEBUG_MESSAGE("ApplicationInstance destroyed.\n");
   destroyed = true;   
}

//-------------------------------------------------------------------------
// provadi cinnost pri aktivaci aplikace
//-------------------------------------------------------------------------
void Application::onActivateApp(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   // Pause if minimized or not the top window
   m_bDDActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
}

//-------------------------------------------------------------------------
// reakce na stisk klavesy v aplikaci
//-------------------------------------------------------------------------
void Application::onKeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
}

//-------------------------------------------------------------------------
// reakce na zpravu, vyzyvajici aplikaci k nastaveni kurzoru
//-------------------------------------------------------------------------
void Application::onSetCursor(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   SetCursor(NULL);
}


//-------------------------------------------------------------------------
// odstranuje herni objekty
//-------------------------------------------------------------------------
void Application::unload()
{   
   DEBUG_MESSAGE("Unload: S\n");
   if (stage != NULL)
   {
      stage->stageError = true;
      stage->stop();
      stage = NULL;
   }
   kbd.setLastScanSet(NULL,0);
   kbd.lastASCIIReset();
   kbd.lastScanReset();

   if (fonts[0]!=NULL) 
   {
      delete (fonts[0]);
      fonts[0] = NULL;
   }
   if (fonts[1]!=NULL) 
   {
      delete (fonts[1]);
      fonts[1] = NULL;
   }
   if (fonts[2]!=NULL) 
   {
      delete (fonts[2]);
      fonts[2] = NULL;
   }
   
   DEBUG_MESSAGE("G");
   SAFE_DELETE(game);   
   
   DEBUG_MESSAGE("M");
   SAFE_DELETE(menu);

   if (toppers != NULL)
   {
      toppers->Save();
      delete toppers;
      toppers = NULL;
   }

   
   if (sound_device != NULL && !sound_device->werror)
      sound_device->SetVolume(vol);
   SAFE_DELETE(sound_device);

}

//-------------------------------------------------------------------------
// zmeni usek hry podle preddefinovaneho 'automatu'
//-------------------------------------------------------------------------
void Application::stateChange(int reason)
{
   if (stage != NULL)
   {
      stage->stageError = false;
      stage->stop();
   }

   switch (state)
   {
	case 0:
	case 3:       
		// menu
      DEBUG_MESSAGE("Finished state 3\n");
      stage = menu;
		menu->start();
      state = 4;
      break;
   case 4:   
		// opoustime menu
      DEBUG_MESSAGE("Finished state 4\n");
      if (reason == REASON_EXIT)
      {
			// bude uplny konec
         state = 13;
         break;
      }
		// ...nebo jdeme do hry
      game->StartGame();
      state = 5;
      break;
   case 5:    
		// zkusime z menu nastartovatlevel
      DEBUG_MESSAGE("Finished state 5\n");
      game->startLevel();
      if (levelname[0] == '\x0')
      {
         state = 3;
      }
      else
      {
         state = 7;
      }
      break;
   case 7:
		// start hry a hra
      DEBUG_MESSAGE("Finished state 7\n");
      stage = game;    
      game->start();
      state = 8;
      break;
   case 8:      
		// byl ukonen level, take znova..
      DEBUG_MESSAGE("Finished state 8\n");
      if (reason == REASON_NEXTLEVEL)
      { 
			game->startLevel();
			game->start();
			state = 8;
         break;
      }
      else if (reason == REASON_THEEND)
      {
         menu->menu = &(menu->victorymenu);
         game->DoneGame();
         state = 3;
			stage = menu;
         break;
      }
      else
      {
         if (reason != REASON_NOTOPPERS)
         {
            menu->menu = &(menu->toppermenu);
			}
         game->DoneGame();
         state = 3;
			stage = menu;
      }
      break;
   case 13:
		// dem dom...
      DEBUG_MESSAGE("Finished state 14\n");
      stage = NULL; 
      PostMessage(m_hWnd,WM_QUIT,0,0);
      break;
   }
}
