#ifndef DX8GRAPH_H_ERIADOR
#define DX8GRAPH_H_ERIADOR

#include <d3d8.h>
#include <d3dx8.h>
#include "tex8manager.h"
#include "types8.h"

//-------------------------------------------------------------------------
// const
//-------------------------------------------------------------------------
#define FMT_A8R8G8B8 1
#define FMT_R8G8B8   2
#define FMT_A1R5G5B5 3
#define FMT_A4R4G4B4 4
#define FMT_R5G6B5   5
#define FMT_COUNT    6

#define MAXMODE      64

//-------------------------------------------------------------------------
// types
//-------------------------------------------------------------------------
   
//-------------------------------------------------------------------------
// global variables
//-------------------------------------------------------------------------
extern HRESULT ddResult;

extern int d3d_ibpp;
extern BYTE d3d_bABits;
extern BYTE d3d_bRBits;
extern BYTE d3d_bGBits;
extern BYTE d3d_bBBits;
extern DWORD d3d_dwAMask;
extern DWORD d3d_dwRMask;
extern DWORD d3d_dwGMask;
extern DWORD d3d_dwBMask;
extern DWORD d3d_dwColorKey;

//-------------------------------------------------------------------------
// macros
//-------------------------------------------------------------------------
/* Problem: tady bylo pouzite d3d_dwBMask, ale ted je mi k nicemu, 
protoze format vga modu nevypovida o formatu textury
#define TO555(x) (((x)&(WORD)d3d_dwBMask)+(((x)&0xFFC0)>>1))
#define TO565(x) (((x)&(WORD)d3d_dwBMask)+(((x)&0x3E0)<<1)+(((x)&0x7c00)<<1))
take vc natvrdo...
*/
#define C565TO555(x) (((x)&(WORD)0x001f)+(((x)&0xFFC0)>>1))
#define C555TO565(x) (((x)&(WORD)0x001f)+(((x)&0x3E0)<<1)+(((x)&0x7c00)<<1))

#define COLOR_KEY565  0xF81F
#define COLOR_KEY555  0x7C1F
#define COLOR_KEY888  0x00FF00FF
//#define COLOR_KEY8888 0xFFFF00FF

//-------------------------------------------------------------------------
// functions
//-------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// DX8Graph - uzitecna trida pro praci s grafikou
//-----------------------------------------------------------------------------
class DX8Graph {
  public:
   LPDIRECT3D8 lpD3D;               // Used to create the D3DDevice
   LPDIRECT3DDEVICE8 lpd3dDevice;   // Our rendering device
   LPD3DXSPRITE lpd3dxSprite;       // Interface to Sprite routines
   D3DDISPLAYMODE d3ddm;            // parametry pouziteho modu
   D3DPRESENT_PARAMETERS d3dpp;     // prezentacni parametry
   D3DCAPS8 d3dcaps;                // Caps for the rendering device
   TextureManager * texturemanager;
   bool fmt_status[FMT_COUNT];
   int picnum;                      // evidenn slo screenshotu (jmno)
   D3DDISPLAYMODE modes16[MAXMODE]; // tabulka pouzitelnych rezimu 16bit
   D3DDISPLAYMODE modes32[MAXMODE]; // tabulka pouzitelnych rezimu 32bit
   int modecount;                   // pocet rezimu (mc16 + mc32)
   int modecount16;                 // pocet rezimu 16 bit
   int modecount32;                 // pocet rezimu 32 bit

	int wide;								// true pokud je 16:10 a vice
	DWORD sizex;							// nastaveno na pouzity x rozmer ozbrazovky
	DWORD sizey;							// nastaveno na pouzity y rozmer ozbrazovky
	DWORD centerx;							// stred x obrazovky
	DWORD centery;							// stred y obrazovky
	double coefx;							// koeficient zvetseni proti klasickemu svga
	double coefy;							// koeficient zvetseni proti klasickemu svga
	double mcoefx;							// koeficient zvetseni pro menu
	double mcoefy;							// koeficient zvetseni pro menu
	DWORD tilesize;						// velikost kostky po pepotu

   // Constructor
   DX8Graph() 
   { 
      lpD3D = NULL; 
      lpd3dDevice = NULL; 
      lpd3dxSprite = NULL;
      texturemanager = NULL;
      picnum = 0;
      modecount = 0;

		wide = 0;
		sizex = 0;
		sizey = 0;
		centerx = 0;
		centery = 0;
		coefx = 0;
		coefy = 0;
		mcoefx = 0;
		mcoefy = 0;

      ZeroMemory(fmt_status, sizeof(fmt_status[FMT_COUNT]));
      //
      texturemanager = new TextureManager();
   }

   // Destructor:
   ~DX8Graph() 
   { 
      if (texturemanager) 
      {
         delete texturemanager; 
         texturemanager = NULL;
      }
      Close();
   }

   // inicializuje grafiku
   HRESULT Init(HWND hWnd, DWORD width = 0, DWORD height = 0, D3DFORMAT format = D3DFMT_UNKNOWN, DWORD refreshrate = 0, bool windowed = FALSE);

   // vsechno to zabali (taky ho vola destruktor graph)
   void Close();

   // zarizuje realokaci pri lost device (release + restore)
   void Reset();

	// pepoet z klasickch bublinovch souadnic na promnn
	long CoordX(long in);

	// pepoet z klasickch bublinovch souadnic na promnn
	long CoordY(long in);

	// pepoet z klasickch bublinovch souadnic na promnn
	RECT Coord(RECT in);

   // likviduje veci, ktere jsou soucasti device (textury, vertexy a tak.)
   HRESULT DeleteDeviceObjects();

   // uvolni veciz device, aby bylo mozne provest reset
   HRESULT ReleaseDeviceObjects();

   // obnovuje veci, ktere posly behem release
   HRESULT RestoreDeviceObjects();

   // blituje sprite
   HRESULT BltSprite(RECT *pDestRect, Texture pSrcTexture, RECT *pSrcRect, 
                     DWORD dwFlags,  D3DCOLOR modulate = 0xFFFFFFFF);
   // vymaklejsi blit
   HRESULT BltSpriteEx(RECT *pDestRect, Texture pSrcTexture, RECT *pSrcRect, 
                       DWORD dwFlags, D3DCOLOR modulate = 0xFFFFFFFF, 
                       float rotation = 0, POINT *prcenter = NULL );
	
	// blit s motion blurem
	HRESULT BltMotionBlur(RECT *pNewRect, RECT *pOldRect, Texture pSrcTexture, RECT *pSrcRect,
								 DWORD dwFlags,  D3DCOLOR modulate = 0xffffffff);

	// plcne obrzek pes celou obrazovku
	HRESULT BltBack(Texture tex, D3DCOLOR modulate = 0xFFFFFFFF);

   // nastavuje pouzitelne formaty textur
   void CheckFormats();

   // zjisti seznam pouzitelnych modu (vraci pocet)
   int GetModeList();

   // zamyka surface
   void LockSurface(Surface surf, D3DLOCKED_RECT* pLockedRect, CONST RECT* pRect = NULL, DWORD flags = 0);

   // zamyka prvni mipmap textury
   void LockTexture(Texture tex, D3DLOCKED_RECT* pLockedRect, CONST RECT* pRect = NULL, DWORD flags = 0);

   // prost screenshot (ulo rendertarget do .bmp)
   void Screenshot();

}; 

//----------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------

extern DX8Graph dx8;


#endif
