/*
  Font8
  by Gunny & Vojvo (kind of)
        
-------------------------------------------------------------------------*/

#ifndef FONT8_H_ERIADOR
#define FONT8_H_ERIADOR

#include "dx8graph.h"
#include "rfile32.h"
#include "tex8manager.h"
#include "types8.h"

#define ALIGN_LEFT 0
#define ALIGN_CENTER 1
#define ALIGN_RIGHT 2

#define CACHE_MAX_SIZE 32     // maximalni pocet polozek (radku) v cachi
#define CACHE_MAX_LEN 128     // maximalni delka radku (ve znacich) v cachi
#define CACHED_RECENTLY 128

#define FONT8_MAXCHAR    255  // pocet znku ve fontu
#define FONT8_MAXFONT     16  // pocet fontu?
#define FONT8_MAXNAME     64

class Font8;

class Font8Cache {
public:
   Font8 * owner;                // pro ktery font8 vlastne dela
   char name[FONT8_MAXNAME];	   // id jmeno
   int size;		               // pocet polozek
   int length;		               // max delka stringu
   DWORD width;		            // x rozmer v pix (pocita se z len.)
   DWORD height;	               // y rozmer v pix (pocita se ze size)
   Texture data;  	            // cacheovana data
   char txt[CACHE_MAX_SIZE][CACHE_MAX_LEN];	// cacheovane texty
   BYTE spacing[CACHE_MAX_SIZE];	// spacing cacheovaneho textu
   int recently[CACHE_MAX_SIZE];	// aktualnost textu
   int pixlen[CACHE_MAX_SIZE];	// pixlen polozek

   Font8Cache()
   {
      owner = NULL;
      *name = 0;
      size = 0;
      length = 0;
      width = 0;
      height = 0;
      data = NULL;
   }
   ~Font8Cache()
   {
      owner = NULL;
      *name = 0;
      size = 0;
      length = 0;
      width = 0;
      height = 0;
      if (data != NULL)
      {
         dx8.texturemanager->Remove(data,true);
         data = NULL;
      }
   }
   HRESULT Init(Font8 * _owner, const char * _name, int _size, int _length);
   HRESULT InitTexture();
   int Find(const char * _txt, BYTE _spc);
   int FindLeastUsed();
};

class Font8 {
public:
   // stav fontu
   bool ready;
   // promenne souvisejici se znaky
   int first;       // kterym znakem bitmap zacina (int je pro jistotu kvuli pretekani)
   int last;        // kterym znakem bitmap konci
   int  width;      // max. delka znaku
   int  height;     // max. vyska znaku (vyska radku)
   Texture data;     // bitmap se znaky
   BYTE rowsize;     // ?
   BYTE xand;
   BYTE yshift;
	BYTE default_spacing;	// pouije se, pokud nen zadna jin hodnota (tj. vol se s 255)
   int  char_width[FONT8_MAXCHAR+1];   // delky jednotlivych znaku
   Font8Cache cache; // cacheovaci objekt

   Font8()
   {
      ready = false;
		default_spacing = 1;
		for (int i=0; i<=FONT8_MAXCHAR; i++) char_width[i] = 0;
   };
   ~Font8() 
   {
      if (data != NULL)
      {
         dx8.texturemanager->Remove(data,true);
         data = NULL;
      }
   };

   DWORD PixLen(const char *_text, BYTE spacing = 255);
   void Load(char * filename, int size = 8, int len = 64);
   int Prepare(RECT * r, const char * what, BYTE spacing = 255);
   void Print(DWORD x, DWORD y, const char * what, DWORD color = 0xffffffff, int align = ALIGN_LEFT, BYTE spacing = 255, DWORD altheight = 0);
   // DWORD colormask, - to prece vyplyva z formatu fontu

   HRESULT Generate(Texture tex, DWORD x, DWORD y, const char * what, BYTE spacing);


   //virtual Surface puts(DWORD _x, DWORD _y, Surface _dest, 
   //   const char *_text, DWORD *_tw = NULL, bool _safe = true) = 0;
};


// loads fonts defined in font2_names
//void loadFonts();


// variables
//extern int font_count;
//extern char *font_names[FONT8_MAXFONT];
extern Font8* fonts[FONT8_MAXFONT];


#endif