#include <stdio.h>
#include "utils.h"
#include "exceptions.h"
#include "framesv.h"


//-------------------------------------------------------------------------
// set
//
// sezvyka retezec s definici chainu:
//   1,2,3,4,&1 ... chain s pokracovanim v chainu 1
//   1,2,3,4    ... chain s pokracovanim od zacatku
//   1,2,@3,4   ... chain s pokracovanim od 3. frame
//-------------------------------------------------------------------------
void FrameChain::set(char *chain)
{
   int done = 0;
   int res = 0;
   int index = 0;  // posunuje se po chainu
   int val = 0;
   int idjump = 0;
   char *p = chain;

   while (!done)
   {
      if (*p == 0)
      {
         done = 1;
      }
      // & - nasleduje id navazujiciho chainu
      if (*p == '&')
      {
         p++;
         idjump = 1;
      }
      // cislice - prepocitavam val
      if (*p >= '0' && *p <= '9')
      {
         val = val*10 + (*p - '0');
      }
      // @ - nastavim loopindex
      if (*p == '@')
      {
         loopindex = index;
         val = 0;
      }
      // dalsi frame
      if (*p == ',' || done)
      {
         if (idjump)
         {
            if (val >= MAX_CHAINS)
            {
               throw new BubException("set: Chain ID greater than max");
            }
            loopindex = -(val+1);
            done = 1;
         }
         else
         {
            if (val != EOC)
            {
               // je to slusne cislo?
               if (val>SAFE_FRAME)
               {
                  throw new BubException("set: Frame ID greater than max");
               }
               // ulozim a posunu na dalsi policko
               this->chain[index] = val;
               index++;
               this->chain[index] = EOC; // dalsi policko zatim neni
               val = 0;
               if (index >= (CHAIN_LEN-1))
               {
                  done = 1;
               }
            }
         }
      }
      // dalsi znak, prosim!
      p++;
   }
}

//-------------------------------------------------------------------------
// jumpNext
//
// posune se na dalsi frame v retezci
//-------------------------------------------------------------------------
void FrameController::jumpNext(int steps)
{
	while (steps > 0)
	{
		frameid = chains[chainid].chain[index];
		looped = false;
		// neni to nahodou konec retezce? to by bylo treba jeste maknout...
		if (frameid == EOC)
		{
			int loopindex = chains[chainid].loopindex;
			if (loopindex < 0)
			{
				// zmenim chain a zacnu od zacatku
				setChain((BYTE)((-loopindex)-1));
				index = 0;
			}
			else
			{
				// posunu se na loopindex a zjistim, co je tam
				looped = true;
				index = loopindex;
			}
			frameid = chains[chainid].chain[index];
		}
		// tak to by bylo...
		index ++; // posuneme pro priste
		steps --;
	}
}
//-------------------------------------------------------------------------
// voli framechain
//-------------------------------------------------------------------------
void FrameController::setChain(BYTE id)
{
   if (id >= MAX_CHAINS)
      throw new BubException("setChain: Chain ID greater than max");
   chainid = id;
   looped = false;
   index = 0;
}

//-------------------------------------------------------------------------
// setDirection
//
// voli smerovou variantu chainu
//-------------------------------------------------------------------------
void FrameController::setDirection(BYTE dir)
{
   if (dir == DIR_L)
   {
      chains = frames->lchains;
      direction = DIR_L;
      index = 0;
      return;
   }
   if (dir == DIR_R)
   {
      chains = frames->rchains;
      direction = DIR_R;
      index = 0;
      return;
   }
   throw new BubException("setDirection: unknown direction!");
}
