/*
 *--------------------------------------------------------------------
 *
 * property
 *
 * nacitani a udrzovani properties pro jednotlive herni objekty
 *
 * verze   0.99.3 - 08.12.1999
 * autori  Gunny, Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny
 *
 * 0.99.3 - tintcolor2 kvuli teleportu  
 * 0.11.3 - pridana vlastnost hitx,hity
 * 0.7.1 - uprava destruktoru, aby byl odolny vuci 
 *         "vicenasobnemu volani"
 * 0.6.1 - rozsireni o nektere dalsi property a zbeautifuleni
 * 0.0.1 - prvni verze

  2009-12-25 / cistka mrtvych polozek (mask, tint)
  2009-12-26 / predano releasovani textur v destruktoru
 *--------------------------------------------------------------------
 */

//#define PROPERTY_DEBUG

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "consts.h"
#include "exceptions.h"

#ifndef PROPERTY_DEBUG
#include "dx8graph.h"
#include "tex8manager.h"
#endif

#include "readini.h"
#include "application.h"
#include "property.h"

void bugreport(int num)
{
#ifdef PROPERTY_DEBUG
   printf("Error at line %i\n",num);
#endif
}

Property::Property()
{
   id = 0;
   xsize = ysize = 40;
   rsize = 8;
   xoff = yoff = 0;
   stablel = 10;
   stabler = 30;
   value1 = value2 = 0;
   surf = NULL;
	bubsurf = NULL;
   fTgs = 50;
   rh = NULL;
	rhb = NULL;
   hitx = PLAYERMONSTER_HITTOLX;
   hity = PLAYERMONSTER_HITTOLY;
	margin = 0;
}


//#########################################################################
//  Properties
//#########################################################################

Properties::Properties()
{
   for (int i=0; i<MAX_ID; i++)
      member[i] = NULL;
   member[0] = new Property();
}

//=========================================================================

Properties::~Properties()
{
   for (int i=0; i<MAX_ID; i++)
   {
      if (member[i] != NULL)
      {
			// zlikviduju textury, ktere jsou odtud alokovane
			if (member[i]->surf) textureManager->Remove(member[i]->surf);
			if (member[i]->bubsurf) textureManager->Remove(member[i]->bubsurf);
			//
         delete member[i];
         member[i] = NULL;
      }
   }
}

//=========================================================================

int Properties::load(char *filename)
{
   FILE        *f;
   Property    *m;
   char        line[MAX_LINE];
   char        var[MAX_LINE];
   char        val[MAX_LINE];
   long        numvalx;
   long        numvaly;
   int         tag;
   int         id = 0;
   int         linenum = 0;   // cislo radku - pro chybove hlaseni

   f = fopen(filename,"rt");
   if (f == NULL)
      return 1;

   while (!feof(f))
   {
      // nastavim si promenne
      m = member[id];
      m->id = id;

      // muzu cist ze souboru
      if (fgets(line, MAX_LINE, f))
      {
         // pocitani radku
         linenum++;  //zatim staci, ne?

         tag = BreakLine(line,var,val);
         NumVal(val,&numvalx,&numvaly);
         // tak co nam radek rika?
         if (tag == 1)
         {
            // je to navesti na dalsi id
            if (numvalx >= 0 && numvalx < MAX_ID)
            {
               // muzu zmenit id
               id = numvalx;
               // mam uz alokovane?
               if (member[id] == NULL)
               {
                  member[id] = new Property();
               }
               m = member[id];
               m->id = id;
            }
            else
            {
               bugreport(linenum);
            }
         }
         else if (tag == 2)
         {
            // prirazeni hodnoty
            // textove parametry
            if (!stricmp(var,"gfx"))
            {
#ifndef PROPERTY_DEBUG
               textureManager-> Load(&(m->surf),val, glSpritesFake);
               m->rh = textureManager->GetHeader(m->surf);
#endif
            }
            else if (!stricmp(var,"bubgfx"))
            {
#ifndef PROPERTY_DEBUG
               textureManager->Load(&(m->bubsurf),val, glSpritesFake); 
               m->rhb = textureManager->GetHeader(m->bubsurf);
#endif
            }
            else if (!stricmp(var,"ch_stand_l"))
            {
               m->frames.lchains[CHAIN_STAND].set(val);
            }
            else if (!stricmp(var,"ch_stand_r"))
            {
               m->frames.rchains[CHAIN_STAND].set(val);
            } 
            else if (!stricmp(var,"ch_walk_l"))
            {
               m->frames.lchains[CHAIN_WALK].set(val);
            }
            else if (!stricmp(var,"ch_walk_r"))
            {
               m->frames.rchains[CHAIN_WALK].set(val);
            } 
            else if (!stricmp(var,"ch_melt_l"))
            {
               m->frames.lchains[CHAIN_MELT].set(val);
            }
            else if (!stricmp(var,"ch_melt_r"))
            {
               m->frames.rchains[CHAIN_MELT].set(val);
            }
            else if (!stricmp(var,"ch_jump_l"))
            {
               m->frames.lchains[CHAIN_JUMP].set(val);
            }
            else if (!stricmp(var,"ch_jump_r"))
            {
               m->frames.rchains[CHAIN_JUMP].set(val);
            } 
            else if (!stricmp(var,"ch_fall_l"))
            {
               m->frames.lchains[CHAIN_FALL].set(val);
            }
            else if (!stricmp(var,"ch_fall_r"))
            {
               m->frames.rchains[CHAIN_FALL].set(val);
            } 
            else if (!stricmp(var,"ch_fire_l"))
            {
               m->frames.lchains[CHAIN_FIRE].set(val);
            }
            else if (!stricmp(var,"ch_fire_r"))
            {
               m->frames.rchains[CHAIN_FIRE].set(val);
            } 
            else if (!stricmp(var,"ch_bubble_g"))
            {
               m->frames.lchains[CHAIN_BUBBLE].set(val);
            }
            else if (!stricmp(var,"ch_bubble_b"))
            {
               m->frames.rchains[CHAIN_BUBBLE].set(val);
            } 
            else if (!stricmp(var,"ch_scrape_g"))
            {
               m->frames.lchains[CHAIN_SCRAPE].set(val);
            }
            else if (!stricmp(var,"ch_scrape_b"))
            {
               m->frames.rchains[CHAIN_SCRAPE].set(val);
            } 
            else if (!stricmp(var,"ch_dying"))
            {
               m->frames.lchains[CHAIN_DYING].set(val);
               m->frames.rchains[CHAIN_DYING].set(val);
            }
            else if (!stricmp(var,"ch_release"))
            {
               m->frames.lchains[CHAIN_RELEASE].set(val);
            }
            else if (!stricmp(var,"ch_blow"))
            {
               m->frames.lchains[CHAIN_BLOW].set(val);
            }
            // ciselne hodnoty
            else if (numvalx >= 0)
            {
               // 2 rozmerne ciselne hodnoty
               if (numvaly >= 0)
               {
                  if (!stricmp(var,"size"))
                  {
                     m->xsize = numvalx;
                     m->ysize = numvaly;
                  }
                  else if (!stricmp(var,"off"))
                  {
                     m->xoff = numvalx;
                     m->yoff = numvaly;
                  }
                  else if (!stricmp(var,"value"))
                  {
                     m->value1 = numvalx;
                     m->value2 = numvaly;
                  }
                  else if (!stricmp(var,"stable"))
                  {
                     m->stablel = numvalx;
                     m->stabler = numvaly;
                  }
                  else if (!stricmp(var,"hit"))
                  {
                     m->hitx = numvalx;
                     m->hity = numvaly;
                  }
               }
               // jednorozmerne
               else
               {
                  if (!stricmp(var,"speed"))
                  {
                     m->speed = numvalx;
                  }
                  else if (!stricmp(var,"ftgs"))
                  {
                     m->fTgs = numvalx;
                  }
                  else if (!stricmp(var,"margin"))
                  {
                     m->margin = numvalx;
                  }
                  else
                  { // netrefil jsem zadnou promennou, takze to musi byt BUG!
                     bugreport(linenum);
                  }
               }
                              
            }
            else
            {
               bugreport(linenum);
            }
         }
      }
   }
   fclose(f);

   return 0;
}
