/*************************************************************************
//                         Bonus class implementation                       
/*************************************************************************
//  Written by : Vojvo (& G.)                                                  
//  Language   : VisuC6 & DX8
//-------------------------------------------------------------------------
// changes:
   0.8.1 - pridany zvuky
   0.10.1 - zvuky stereo
   2008-06-20 / pekopno a zaremovno
   2008-06-28 / postupn mizen
	2009-12-25 / drobny uklid stareho kodu
*/


#include <stdlib.h>
#include "dx8graph.h"
#include "exceptions.h"
#include "utils.h"
#include "wsound.h"
#include "gs_game.h"
#include "sprites.h"

#define BONUS_TIME 900

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bonus::Bonus (int onx, int ony, Property *prop) : Sprite(onx,ony,prop)
{
   value = prop->value1;
   movex = -BONUS_MOVEX1 + random(BONUS_MOVEX2);
   movey = -BONUS_MOVEY - random(BONUS_MOVEY);
   counter = 0;
   action = SPMA_JUMP;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Bonus::~Bonus()
{
}

//-------------------------------------------------------------------------
// vykreslovani bonusu
//-------------------------------------------------------------------------
void Bonus::Paint()
{
	/*
   int y = newy;
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
   if (newy <= PLAYFIELD_MIN_Y)
   {
      frameRect.top = frameRect.bottom - (newy - b_miny);
      y = PLAYFIELD_MIN_Y;
      clipped = true;
   }
   else if (newy >= b_maxy)
   {
      clipped = true;
      frameRect.bottom = frameRect.top + PLAYFIELD_MAX_Y-newy;
      ybottom = PLAYFIELD_MAX_Y-1;
   }
   else if (clipped)
   {
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + xsize;  
      frameRect.bottom = frameTop + ysize;  
      clipped = false;
   }
   if (!clipped || frameRect.bottom > frameRect.top)
   {
		
		RECT r;
		r.top = y;
		r.left = newx;
		r.bottom = y + (frameRect.bottom-frameRect.top);
		r.right = newx + (frameRect.right-frameRect.left);
		r = dx8.Coord(r);
		//putcel(newx,newy,dest,frameRect,gfx);
	*/
		RECT r;
		r.top = newy; //y
		r.left = newx;
		r.bottom = r.top + ysize; // +(frameRect.bottom-frameRect.top);
		r.right = r.left + xsize; // +(frameRect.right-frameRect.left);
		r = dx8.Coord(r);
	
		D3DCOLOR colmask = 0xffffffff;
		// osetreni postupneho mizeni
		int rest = (BONUS_LIFE - counter);
		if (rest < 1024)
		{
			rest = rest / 4;
			rest = rest << 24;
			colmask &= 0x00ffffff;
			colmask |= rest;
		}
		mask = colmask;
		
		dx8.BltSprite(&r, gfx, &frameRect, 0, mask);
		/*
   }
*/
}


//-------------------------------------------------------------------------
// pohyb bonusu
//-------------------------------------------------------------------------
int Bonus::Move()
{
  int result = 0;

  counter++;
  mul_x += movex;
  newx = mul_x / MUL_MOVE;
  if (newx<PLAYFIELD_MIN_X) {
    newx = PLAYFIELD_MIN_X;
    mul_x = newx * MUL_MOVE;
    movex = -movex;
    result |= SP_COL_L;
  }
  else if (newx>b_maxx) {
    newx = b_maxx;
    mul_x = newx * MUL_MOVE;
    movex = -movex;
    result |= SP_COL_R;
  }
  mul_y += movey;
  newy = mul_y / MUL_MOVE;
  if (newy<PLAYFIELD_EMIN_Y) {
    newy = eb_maxy;
    frameRect.top = frameRect.bottom - ysize;
    mul_y = newy * MUL_MOVE;
    result |= SP_COL_U;
  }
  else if (newy>eb_maxy) {
    newy = PLAYFIELD_EMIN_Y;
    frameRect.bottom = frameRect.top + ysize;
    mul_y = newy * MUL_MOVE;
    result |= SP_COL_D;
  }
  if (action & SPMA_JUMP) {
    Haste(0,gravity);
  }

  return result;
}

//-------------------------------------------------------------------------
// reakce na pozici a okoli bonusu
//-------------------------------------------------------------------------
void Bonus::MapReflex()
{
   // zmeny stavu!
   if (counter >= BONUS_TIME)
   {
      // osetreni dopadu na plosinku
      if ((action & SPMA_JUMP) && IsStable() > 0 && newy > PLAYFIELD_MIN_Y)
      {
         sound_device->Play(2,newx+xsize/2);
         movex = 0;
         newy = (int)((newy-40+ysize)/MAP_CELY) * MAP_CELY+40-ysize;
         mul_y = newy * MUL_MOVE;
         movey = 0;
         action ^= SPMA_JUMP;
      }
      // osetreni sezratelnosti
      type = BONUS;
   }
}
