/*
//****************************************************************************
//                        Teleport class implementation
//****************************************************************************
//  Written by : Vojvo (& G.)
//  Language   : VisuC6 + DX6.1 + DX8
// changes:

   0.7.1  - opravena delka jednoho nextlvl
          - pridan destruktor pro odstraneni nacacheovaneho tintu
   0.8.1  - pridan zvuk
   0.99.3 - uprava zobrazovani teleportu
   0.99.5 - teleporty se pri red alertu zobrazuji "redalertove"
          - hraci mizi ve final teleportech
   1.1.2  / zablokovno provdn prepareDest u FTPORTU

  2008-06-22 / pedlno na DX8
  2008-06-22 / polovin prhlednost
  2008-06-22 / zase jinak - vstup je normln, vstup polovin (+nov gfx)
  2009-12-25 / cistka v tintech

*/

#include <string.h>
#include <stdio.h>
#include "exceptions.h"
#include "types8.h"
#include "dx8graph.h"
#include "tex8manager.h"
#include "wsound.h"
#include "gs_game.h"
#include "sprites.h"

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Teleport::Teleport(int onx, int ony, Property *prop)
{
   type = TPORT;
   id = prop->id;
   fTgs = prop->fTgs;
   direction = DIR_L;
	/*
   surfw = prop->rh->dwWidth; 
   surfh = prop->rh->dwHeight;
	*/
   celw = prop->rh->dwCelWidth;
   celh = prop->rh->dwCelHeight;
   celr = prop->rh->byCelRow;
   xsize = prop->xsize;
   ysize = prop->ysize;
   xoff = prop->xoff;
   yoff = prop->yoff;
   offset_dest = (BYTE)prop->value1;
   frames.frames = &(prop->frames);
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   jumpNext();   
   gfx = prop->surf;
   collision = 0;
   gravity = DFT_GRAVITY;
   newx = oldx = onx;
   newy = oldy = ony;
   mul_x = newx << SHIFT_MOVE + MUL_HALF;
   mul_y = newy << SHIFT_MOVE + MUL_HALF;
   movex = movey = 0;
   stable_l = prop->stablel;
   stable_r = prop->stabler;
   topx = prop->speed;
   topy = JUMP_STRENGTH;
   bottomx = -topx;
   bottomy = -topy;
   sort_priority = 0;
   is_stable = 0;
   redden = 0;
}


//---------------------------------------------------------------------------
// Vykresluje teleport
//---------------------------------------------------------------------------
void Teleport::Paint()
{
	RECT r;
	
	// vstup
	r.top = newy;
	r.left = newx;
	r.bottom = r.top + ysize;
	r.right = r.left + xsize;
	r = dx8.Coord(r);
   dx8.BltSprite(&r, gfx, &frameRect, 0, 0xffffffff);

	// vystup
	r.left = destx;
	r.top = desty;
	r.right = r.left + xsize;
	r.bottom = r.top + ysize;
	r = dx8.Coord(r);
   dx8.BltSprite(&r, gfx, &frameRect, 0, 0x80808080);

}

//-------------------------------------------------------------------------
// posunuje se na dalsi frame a nastavuje okraje celu v surface s grafikou
//-------------------------------------------------------------------------
void Teleport::jumpNext()
{
   int l,t;
   frames.jumpNext();                       
   l = (frames.frameid % celr) * celw + xoff;
   t = (frames.frameid / celr) * celh + yoff;
   frameRect.left = l;
   frameRect.top = t;
   frameRect.right = l + celw;  
   frameRect.bottom = t + celh;
   if (type != FTPORT)
   {
      l = ((offset_dest - frames.frameid) % celr) * celw + xoff;
      t = ((offset_dest - frames.frameid) / celr) * celh + yoff;
      destRect.left = l;
      destRect.top = t;
      destRect.right = l + celw;  
      destRect.bottom = t + celh;
   }
}


//-------------------------------------------------------------------------
// transportuje pasazera
//-------------------------------------------------------------------------
void Teleport::Transport(Creature *passenger)
{
   sound_device->Play(11);
   if (type == FTPORT)
   {
      ((Player *) passenger)->sheet.AddPoints(500);
      strcpy(levelname,nextlvls+9*destx);
      player1->state |= INBUBBLE;
      player2->state |= INBUBBLE;
      ARRAY_REMOVE(monsters,monsters_ctrl,player1);
      ARRAY_REMOVE(monsters,monsters_ctrl,player2);
      exitgame = true;
		/*
      vanish_full = VANISH_FADE_TIME / 4; //*** puvodne jen 2 cykly, ale dejme tomu
		vanish = vanish_full;
		*/
		vanish = 1;
		fader.Set(-2);
      return;
   }
   passenger->newx = destx;
   passenger->newy = desty;
   passenger->mul_x = passenger->newx << SHIFT_MOVE + MUL_HALF;
   passenger->mul_y = passenger->newy << SHIFT_MOVE + MUL_HALF;
}

//-------------------------------------------------------------------------
// zjistuje kolize s objekty
//-------------------------------------------------------------------------
int Teleport::CollideWith(Sprite *what)
{
   if (abs(newx-what->newx) < (what->xsize/2))
      if (abs(newy-what->newy) < (what->ysize/2))
         return 1;
      return 0;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Teleport::~Teleport()
{
}