//*************************************************************************
//       Unidentified flying oblect related classes implementation          
//*************************************************************************
//  Written by : Vojvo & Gunny
//  Language   : VisuC6 + DX6.1
/*
// changes
   2008-06-22 / marny pokus u jiny blend mod fireballu
   1.0.2 / cachry s reakci Murrayho na hrace v bubline
// 1.0.1 - snaha o prvni plnou verzi
// 0.10.1 - zvuky stereo
// 0.9.5 - fireball
// 0.9.3 - zprovoznen meat
*/

#include <stdio.h>
#include <stdlib.h>
#include "consts.h"
#include "types8.h"
#include "exceptions.h"
#include "utils.h"
#include "dx8graph.h"
#include "wsound.h"
#include "map.h"
#include "sprites.h"
#include "gs_game.h"


#define MURRAY_ACCELDIV    10

UFO *UFOs[MAX_UFOS] = { NULL, NULL, NULL, NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL };

#ifdef MURRAYDEBUG
char murrayg_msg[256];
char murrayb_msg[256];
#endif

int UFOcnt = 0;


//##########################################################################
//----------------------------------- UFO ----------------------------------
//##########################################################################

int UFO::Move(void)
{
   if (melting)
      return OUT_OF_BOUNDS;
   
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   if (newx < VGA_MIN || newx > (VGA_MAX_X-xsize) || 
      newy < VGA_MIN || newy > (PLAYFIELD_MAX_Y-ysize))
      return OUT_OF_BOUNDS;
   
   return IN_BOUNDS;
}

//--------------------------------------------------------------------------

void UFO::Paint()
{
   //int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
	RECT r;
	r.top = newy;
	r.left = newx;
	r.bottom = newy + ysize;
	r.right = newx + xsize;
	r = dx8.Coord(r);
   //putcel(newx,newy,dest,frameRect,gfx);
   //dx8.BltSprite(&r, gfx, &frameRect, 0);
	dx8.BltMotionBlur(&r, &lastPos, gfx, &frameRect, 0);
	lastPos = r;
}

//--------------------------------------------------------------------------

int UFO::CollideWith(Player *p)
{
   int xc,yc,xlb,xrb,ytb,ybb;
   
   if (melting)
      return 0;
   xc = p->newx - newx;
   yc = p->newy - newy;
   xlb = UFOPLAYER_HITTOLX - p->xsize;
   xrb = xsize - UFOPLAYER_HITTOLX;
   ybb = ysize - UFOPLAYER_HITTOLY;
   ytb = UFOPLAYER_HITTOLY - p->ysize;
   if (xc < xrb && xc > xlb && yc < ybb && yc > ytb)
   {
      Melt();
      p->Die();
      return 1;
   }
   return 0;
}

//--------------------------------------------------------------------------

void UFO::Melt()
{
   melting = TRUE;
}

//##########################################################################
//-------------------------------- Fireball --------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Fireball::Fireball(int onx, int ony, Property *prop, Fireable *par, int spd)
: UFO(onx,ony,prop,par) 
{ 
   movex = spd;
   if (movex > 0)
   {
      direction = DIR_R;
      frames.setDirection(DIR_R);
   }
   else
   {
      direction = DIR_L;
      frames.setDirection(DIR_L);
   }
   jumpNext();
}

//--------------------------------------------------------------------------
int Fireball::Move()
{
   mul_x += movex;
   mul_y += movey;
   newx = mul_x >> SHIFT_MOVE;
   newy = mul_y >> SHIFT_MOVE; 
      
   if (newx < PLAYFIELD_MIN_X || newx > PLAYFIELD_MAX_X-xsize)
      return OUT_OF_BOUNDS;
        
   if (melting) {
      if (frames.looped)
         return OUT_OF_BOUNDS;
      else
         return IN_BOUNDS;
   }
          
   if (!CanGoLR()) {
      sound_device->Play(10,newx+xsize/2);
      Melt();
   }
            
   return IN_BOUNDS;
}
              
//--------------------------------------------------------------------------
int Fireball::CanGoLR()
{
   if (movex > 0)
   {
      if (map_value(newx+24,newy+6,map))
         return 0;
      if (map_value(newx+24,newy+14,map))
         return 0;
   }
   else
   {
      if (map_value(newx+6,newy+6,map))
         return 0;
      if (map_value(newx+6,newy+14,map))
         return 0;
   }
   return 1;
}
                  
//---------------------------------------------------------------------------
void Fireball::Melt()
{
   melting = TRUE;
   frames.setChain(CHAIN_MELT);
   if (direction == DIR_R)
   {
		movex = MOVE50;
      mul_x += MOVE150;
   }
   else
   {
      movex = -MOVE50;
      mul_x -= MOVE150;
   }
   newx = mul_x >> SHIFT_MOVE;
   if (s_drops) 
      drops->HorizontalBlast(newx+5,newy+5,DROP_RED,gravitation,
         (direction == DIR_R)?1:-1);

}

//---------------------------------------------------------------------------
void Fireball::Paint()
{
	//*** Hm, na kraviny to asi bude teba pedlat na vertexy
	/*
    dx8.lpd3dDevice->SetRenderState(D3DRS_BLENDOP ,D3DBLENDOP_ADD);
	dx8.lpd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
	dx8.lpd3dDevice->SetRenderState(D3DRS_SRCBLEND ,0 );
	*/
	 UFO::Paint();
	 /*
    dx8.lpd3dDevice->SetRenderState(D3DRS_BLENDOP ,D3DBLENDOP_ADD);
	 */
}


//##########################################################################
//--------------------------------  Vejce   --------------------------------
//##########################################################################

//size = 28,24
#define HALF_EGG_X 14
#define HALF_EGG_Y 13

int Egg::Move()
{
   movey += gravity;
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   
   if (newy < VGA_MIN || newy > (PLAYFIELD_MAX_Y-ysize))
      return OUT_OF_BOUNDS;
   
   if (melting) {
      if (frames.looped)
         return OUT_OF_BOUNDS;
      else
         return IN_BOUNDS;
   }
   
   if (!CanGoLR()) {
      Melt();
   }
   
   return IN_BOUNDS;
}

//--------------------------------------------------------------------------

int Egg::CanGoLR() //ve skutecnosti spis CanGoDown =)
{
   this_tile = map_value(newx+HALF_EGG_X,
      newy+(2*HALF_EGG_Y-2),map);
   if (this_tile && !last_tile)
   { // dopad na plosinku
      int plat = (newy + ysize) % MAP_CELY;
      newy = newy - plat;
      mul_y = newy << SHIFT_MOVE;
      return 0;
   }
   else
   {
      last_tile = this_tile;
      return 1;
   }
}

//---------------------------------------------------------------------------

void Egg::Melt()
{
   sound_device->Play(32,newx+xsize/2);
   gravity = 0;
   movey = 0;
   melting = TRUE;
   frames.setChain(CHAIN_MELT);
   if (s_drops) 
   {
      drops->Blast(newx+HALF_EGG_X,newy+HALF_EGG_Y,DROP_YELLOW,gravitation);
   }
}


//##########################################################################
//-------------------------------  Masicko  --------------------------------
//##########################################################################

int Meat::Move()
{
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   
   movey += gravity;
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;

   if (newx < PLAYFIELD_MIN_X || newx > (PLAYFIELD_MAX_X-xsize) 
       || newy < PLAYFIELD_MIN_Y || newy > (PLAYFIELD_MAX_Y-ysize))
      return OUT_OF_BOUNDS;
  
   if (melting) 
      return OUT_OF_BOUNDS;

   return IN_BOUNDS;
}
    
//--------------------------------------------------------------------------
int Meat::CanGoLR() //ve skutecnosti spis CanGoDown =)
{
   this_tile = map_value(newx+4,newy+8,map);
   if (this_tile && !last_tile)
   { 
      return 0;
   }
   else
   {
      last_tile = this_tile;
      return 1;
   }
}
       
//---------------------------------------------------------------------------
void Meat::Melt()
{
   sound_device->Play(31,newx+xsize/2); // plesk
   gravity = 0;
   movex = 0;
   movey = 0;
   melting = TRUE;
}

//##########################################################################
//--------------------------------  Murray  --------------------------------
//##########################################################################

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Murray::Murray(int onx, int ony, Property *prop, Player * plr) 
   : UFO(onx,ony,prop,NULL)
{
   target=plr;
   targx = 320;
   targy = 240;
   speed = prop->speed;
   maxspeed = 300;
   accelcounter = 0;
	
	// hack, ale snad to nebude dlat bordel
	haloRect.left = 0;
	haloRect.top = 2 * celh;
	haloRect.right = celw;
	haloRect.bottom = 3* celh;
	haloAlpha = 0x00ffffff;

}

//--------------------------------------------------------------------------
int Murray::Move()
{
	// Nastaveni sledovaci pozice
	/*
   if (!((target->state) & (INVISIBLE+INBUBBLE)) || (target->action & SPMA_FIRE))
   {
      targx = target->newx;
      targy = target->newy;
   }
	*/
	// V. 2004 - predelano
	// melo by to updatovat jenom tehdy, kde je hrac videt a mimo bublinu
	// v bubline by to melo updatovat jen v pripade, ze vrze
	// pokud byl hrac v bubline uz pri objeveni Murrayho, 
	// tak by mel hlasit pozici, kde ho sezrala bublina
	// (To vsechno se ale resi uz u hrace.)
	targx = target->lastseenx + 6; // +6 je hack korigujici ruznou sirku hrace a M.
	targy = target->lastseeny + 9; // +9 totez pro vysku

   // nejdriv obe zrychleni
   accx = targx-newx;
   accy = targy-newy;
   //if (target->repelent && (abs(accx)+abs(accy)) < target->repelent)
	if (target->repelent && (accx*accx + accy*accy) < (target->repelent*target->repelent))
   {
      accx = -accx;
      accy = -accy;
   }
   //X
   movex += accx >> 2;
   if (movex > 0)
   {
      if (movex > maxspeed) 
         movex = maxspeed;
   }
   else
   {
      if (movex<(-maxspeed)) 
         movex=(-maxspeed);
   }
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   //Y
   movey += accy >> 2;
   if (movey > 0)
   {
      if (movey > maxspeed) 
         movey = maxspeed;
   }
   else
   {
      if (movey<(-maxspeed)) 
         movey=(-maxspeed);
   }
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   //SPEED
   accelcounter++;
   if (maxspeed<speed && accelcounter == MURRAY_ACCELDIV) 
   {
      maxspeed += 1;
      accelcounter = 0;
   }
   //odrazy
   if (newx < PLAYFIELD_MIN_X)
   { 
      newx = PLAYFIELD_MIN_X;
      movex = -movex;
   } 
   else if (newx > (PLAYFIELD_MAX_X-xsize))
   { 
      newx = PLAYFIELD_MAX_X-xsize;
      movex = -movex;
   }
   if (newy < PLAYFIELD_MIN_Y)
   { 
      newy = PLAYFIELD_MIN_Y;
      movey = -movey;      
   } 
   else if (newy > (PLAYFIELD_MAX_Y-ysize))
   { 
      newy = PLAYFIELD_MAX_Y-ysize;
      movey = -movey;      
   }
#ifdef MURRAYDEBUG
   sprintf(pom_string,"%d:%d,%d",maxspeed,movex,movey);
   if (id == ID_MURRAYG)
      strcat(murrayg_msg,pom_string);
   else
      strcat(murrayb_msg,pom_string);
#endif
   //aby se zlikvidoval po rozprsknuti
   if (melting) {
      if (frames.looped)
         return OUT_OF_BOUNDS;
      else
         return IN_BOUNDS;
   }
   
   return IN_BOUNDS;
}

//--------------------------------------------------------------------------

void Murray::jumpNext(int steps)
{
	Sprite::jumpNext(steps);
	
	haloRect.left += celw;
	haloRect.right += celw;
	if (haloRect.left >= (celw * 4))
	{
		haloRect.left = 0;
		haloRect.right = celw;
	}
	
	if (haloAlpha <= 0xa0ffffff) haloAlpha += 0x04000000;

}

//--------------------------------------------------------------------------

void Murray::Paint()
{
	RECT r;

	// ze
	if (frames.frameid<=2)
	{
		// jen kdy je to normln frame (ili neobjevuje se ani nerozplv)
		r.top = newy - ysize/2;
		r.left = newx - xsize/2;
		r.bottom = newy + ysize + ysize/2;
		r.right = newx + xsize + xsize/2;
		r = dx8.Coord(r);
		
		dx8.BltSprite(&r, gfx, &haloRect, 0, haloAlpha); //0x80ffffff
	}

	// sprite
	r.top = newy;
	r.left = newx;
	r.bottom = newy + ysize;
	r.right = newx + xsize;
	r = dx8.Coord(r);
   //dx8.BltSprite(&r, gfx, &frameRect, 0);
	dx8.BltMotionBlur(&r, &lastPos, gfx, &frameRect, 0);
	lastPos = r;

}
      
//--------------------------------------------------------------------------
        
void Murray::Melt()
{
   if (!melting)
   {
      melting = TRUE;
      frames.setChain(CHAIN_MELT); //skok na rozprsk
   }
}
