//**************************************************************************
//                 Bubble related classes implementation                  
//**************************************************************************
// Written by : Vojvo & Gunny    
// Language   : Visuc6 + DX6.1  
/*

   2011-09-04 / symbub nemnily velikost
   2008-06-22 / size rect mus bt 20, dn -1 tam nepat
   2008-06-20 / pedlno na DX8
// 0.10.3 - bubliny pouzivaji kopii backgroundu v pameti
// 0.10.2 - plne a prazdne bubliny praskaji s jinym samplem 
// 0.9.3 - skrabani na bublinu zevnitr
//       - domino pusobi i na symbolove bubliny
// 0.8.2 - pridany zvuky
// 0.8.1 - pridany symbolove bubliny
// 0.7.2 - opraven bug se spatnym volanim jumpNext z Bub konstruktoru u 
//         Release buildu
*/

#include <stdio.h>
#include "gonio.h"
#include "exceptions.h"
#include "utils.h"
#include "dx8graph.h"
#include "map.h"
#include "wsound.h"
#include "drops.h"
#include "gs_game.h"
#include "application.h"
#include "sprites.h"

#define INERTIA_FIRE 18
#define INERTIA_NORM 2


//##########################################################################
//---------------------------------- Bub -----------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bub::Bub (int onx, int ony, Property *prop)
{
   type = BUBBLE;
   id = prop->id;
   fTgs = prop->fTgs;
   if (id == ID_BUBBLE || id == ID_SBUBBLE)
   {
		// sklenene a symbolove bubliny, doufam (V. 2006)
		/*
      xsize = glasses[0]->GetWidth();
      ysize = glasses[0]->GetHeight();
		*/
   }
   else
   {
		// vsechny ostatni - obrysove bubliny?
      xsize = prop->xsize;
      ysize = prop->ysize;
      gfx = prop->surf;
      celw = prop->rh->dwCelWidth;
      celh = prop->rh->dwCelHeight;
      celr = prop->rh->byCelRow;
   }
   b_maxy = PLAYFIELD_MAX_Y - ysize;
   b_maxx = PLAYFIELD_MAX_X - xsize;
   frames.frames = &(prop->frames);
   direction = DIR_L;
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   collision = 0;
   newx = oldx = onx;
   newy = oldy = ony;
   mul_x = newx << SHIFT_MOVE + MUL_HALF;
   mul_y = newy << SHIFT_MOVE + MUL_HALF;
   movex = movey = 0;
   topx = prop->speed;
   topy = prop->speed;
   bottomx = -topx;
   bottomy = -topy;
   sort_priority = 0;
   is_stable = 0;
   clipped = false;
   counter = BUB_LIFETIME;
   state = 0;
   sender = NULL;
   computed = false;
}

//-------------------------------------------------------------------------
// vraci TRUE, pokud je bublina pomala
//-------------------------------------------------------------------------
int Bub::IsSlow()
{
   return (movex >= bottomx && movex <= topx);
}

//==========================================================================
int Bub::Move()
{
   int result = 0;
   
   SlowIt(INERTIA_NORM,INERTIA_NORM);

   // vitr a zpomalovani rychle bubliny
   if (movex >= bottomx && movex <= topx)
   {
      if (windmap != NULL)
      {
         movex += wndmap_value(newx,newy-PLAYFIELD_MIN_Y,windmap);
         if (movex > topx) movex = topx;
         if (movex < bottomx) movex = bottomx;
      }    
   }
   else
      SlowIt(INERTIA_FIRE,0);
   
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   if (newx < PLAYFIELD_MIN_X) {
      newx = PLAYFIELD_MIN_X;
      mul_x = newx << MUL_MOVE;
      movex = -movex;
      result |= SP_COL_L;
   }
   else if (newx > b_maxx) {
      newx = b_maxx;
      mul_x = newx << SHIFT_MOVE;
      movex = -movex;
      result |= SP_COL_R;
   }

   // vitr v y-smeru
   if (movex >= bottomx && movex <= topx && windmap != NULL)
   {
      movey += wndmap_value(newx,newy-PLAYFIELD_MIN_Y,windmap+WNDMAP_SIZE);
      if (movey > topy) movey = topy;
      if (movey < bottomy) movey = bottomy;
   }
   
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   if (newy < PLAYFIELD_MIN_Y) {
      newy = PLAYFIELD_MIN_Y;
      mul_y = newy << SHIFT_MOVE;
      movey = -movey;
      result |= SP_COL_D;
   }
   else if (newy>b_maxy) { 
      newy = b_maxy;
      mul_y = newy << SHIFT_MOVE;
      movey = -movey;
      result |= SP_COL_U;
   }
   
   counter--;
   if (counter == BUB_BLOWTIME)
   {
      result |= BUB_BLOWSTART;
   }
   
   if (!counter)
   {
      result |= BUB_BLOWEND;
   }
   
   return result;
}

//==========================================================================
int Bub::Domino()
// Dominovy efekt
// otestuje, jestli jsou pobliz nejake pomale nebo symbolove bubliny
// pokud jsou, nastavi je tesne pred prasknuti a jako sender nastavi
// stejny pointer jako u bubliny, ktera prave praskla
// vraci pocet bublin, ktere exploze ovlivnila
{
   int res=0;
   //nejdriv otestuju normalni bubliny
   int bi=0;
   while (bi < sbubbles_ctrl.count)
   {
      Bubble *pom_b = ARRAY_AT(sbubbles,bi);
      int ax = abs(newx-pom_b->newx);
      if ((ax < 44) && (abs(newy-pom_b->newy) < ycircle40[ax]))
      {
         if (pom_b->counter>BUB_DOMINATED)
         {
            res++;
            pom_b->counter=BUB_DOMINATED;
            pom_b->sender=sender;
         }
      }
      bi++;
   }
   //ted zkusim, co to udela se symbolovyma...
   bi=0;
   while (bi < symbubs_ctrl.count)
   {
      SymbolBubble * pom_s = ARRAY_AT(symbubs,bi);
      int ax = abs(newx-pom_s->newx);
      if (ax < 44 && abs(newy-pom_s->newy) < ycircle40[ax])
      {
         if (pom_s->counter>BUB_DOMINATED)
         {
            res++;
            pom_s->counter=BUB_DOMINATED;
            pom_s->sender=sender;
         }
      }
      bi++;
   } 

   return res;
}

//===========================================================================
int Bub::CollideWith(Sprite *what)
// Testovani kolizi a rozstrakavani
{
   int result,sx,sy,ax,ay,sgnx,sgny;
   WORD normx, normy;
   int diff_x,diff_y;
   
   if (state & BLOWING)
      return 0;
      
   result = 0;
   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   switch (what->type) {
   case PLAYER: if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                   diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                  if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                      diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))                  
                      if (movex >= bottomx && movex <= topx)
                         result = 1;
                      break;
   case BUBBLE: sx = newx - what->newx;
      sy = newy - what->newy;
      ax = abs(sx);
      ay = abs(sy);
      sgnx = (sx > 0) ? 1:-1;
      sgny = (sy > 0) ? 1:-1;
      if (ax < 32 && ay < ycircle32[ax])
      {
         normx = normx50(ax,ay);
         normy = normy50(ax,ay);
         movex = sgnx * normx;
         movey = sgny * normy;
         what->movex = (-sgnx) * normx;
         what->movey = (-sgny) * normy;
      }
      result = 1;
   }
   return result;
}


//##########################################################################
//-------------------------------- Bubble ----------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bubble::Bubble(int onx, int ony, Property *prop) 
: monster(NULL), whose(NULL), Bub (onx,ony,prop)
{
   nextscrape = 0xFFFF;
   selfblowing = 0;
   jumpNext();   
}

//-------------------------------------------------------------------------
// testovani kolizi
//-------------------------------------------------------------------------
int Bubble::CollideWith(Sprite *what)
{
   int result;
   int diff_x,diff_y;
   
   if (state & BLOWING)
      return 0;
   
   result = 0;
   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   switch (what->type) {
   case MONSTER: if (!monster && counter < BUB_START)
                    if (movex < bottomx || movex > topx)
                       if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                           diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                          if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                              diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))
                             result = 1;
                 break;
   case PLAYER: if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                   diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                  if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                      diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))
                     if (movex >= bottomx && movex <= topx)
                         result = 1;
                     else
                        if (!monster)
                        {
                           Player *pom_p = (Player *)what;
                           if (!(pom_p->state & (BUBBLE_RESISTANT)))
                           {
                              if (((movex > 0) ^ (pom_p->direction == DIR_R)) 
                                 || (s_catchy && whose != pom_p))
                              {
                                 pom_p->state |= INBUBBLE;
                                 result = 2;
                              }
                           }
                        } 
                break;
   case BUBBLE: result = Bub::CollideWith(what);
   }
   return result;
}

//===========================================================================
int Bubble::Move()
{
   int result;
   result = Bub::Move();
   
   // praskla-li bublina, vratime energii
   if (result & BUB_BLOWEND && whose)
   {
      if (whose->sheet.energy<whose->sheet.maxenergy)
      {
         whose->sheet.energy++;
      }
   }

   // skrabani zevnitr na bublinu
   if (monster)
   {
      if (monster->updateable)
      {
         monster->BubbleAction(newx,newy);
      }
      
      if (counter < nextscrape && monster->type == PLAYER)
      {
         monster->KeyReflex();
         Player * pmon = (Player *)monster;
         if (pmon->state & WANNALEAVE)
         {
            pmon->state ^= WANNALEAVE;
            pmon->frames.setChain(CHAIN_SCRAPE);
            if (counter > BUB_LASTSCRAPE)
            {
               sender = pmon;
               counter -= BUB_SCRAPEEFFECT;
               if (counter > BUB_SCRAPELENGTH)
                  //nextscrape = counter-(BUB_SCRAPELENGTH+3-((pmon->acceleration/3)); //pokud je hbitejsi, vrze rychleji
                  nextscrape = counter-BUB_SCRAPELENGTH;
               else
                  nextscrape = 0;
               sound_device->Play(8);
            }
            else
            {
               counter = BUB_BLOWTIME+1;
               nextscrape = 0;
            }
         }
      }
   }
   if ((counter == BUB_BLOWCOME1 || counter == BUB_BLOWCOME2) 
       && monster && monster->type == MONSTER)
   {
      sound_device->Play(8,newx+xsize/2);
      monster->frames.setChain(CHAIN_SCRAPE);
   }

   return result;
}

//===========================================================================
void Bubble::jumpNext(int steps)
{
   // animace bubliny
   frames.jumpNext();
	
	// a pokud to neni normalni sklenena bublina, tak se prepocitavaji nejake okraje??? (V. 2006)
	// (a zajimave je, e i u potvory...? - asi kvuli zpusobu vykreslovani
   if (id != ID_BUBBLE)
   {
      frameLeft = (frames.frameid % celr) * celw;
      frameTop = (frames.frameid / celr) * celh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + celw;  //xsize
      frameRect.bottom = frameTop + celh;  //ysize
		
		// animace potwurky
		if (monster)
			monster->jumpNext();
		
			/*
			// tohle ted snad dela monster->jumpNext(); ne?
			if (monster != NULL)
			{
         monster->frameRect.left = monster->frameLeft;
         monster->frameRect.top = monster->frameTop;
         monster->frameRect.right = monster->frameLeft + monster->celw;  
         monster->frameRect.bottom = monster->frameTop + monster->celh;  
			}
		*/
   }
}

//===========================================================================
void Bubble::Paint()
{
   //int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
	RECT tr;
	tr.top = newy;
	tr.left = newx;
	tr.bottom = newy + ysize;
	tr.right = newx + xsize;
	tr = dx8.Coord(tr);
	
	// obrysove
   if (id != ID_BUBBLE)
   {
      if (monster != NULL)
      {
         //putcel(newx,newy,dest,monster->frameRect,monster->bubgfx);
			dx8.BltSprite(&tr, monster->bubgfx, &(monster->frameRect), 0);
      }
		//*** a co selfblowing? aka kehk
      //putcel(newx,newy,dest,frameRect,gfx);
		dx8.BltSprite(&tr, gfx, &frameRect, 0);
      oldx = newx;
      oldy = newy;
      return;
   }
  
}

//-------------------------------------------------------------------------
// akce pri chyceni cehosi do bubliny
//-------------------------------------------------------------------------
void Bubble::Catch(Creature *monster)
{
   mon_chain = monster->frames.chainid;
   mon_dir = monster->direction;
   this->monster = monster;

	// hack na rozlien frejm potvory podle barvy bubliny
	/*
   if (whose == player1)
   {
      monster->direction = DIR_L;
      monster->frames.setDirection(DIR_L);
   }
   else
   {
      monster->direction = DIR_R;
      monster->frames.setDirection(DIR_R);
   }
	*/

	// barvu by te mla obstarat samotn bublina, take nechme natvrdo DIR_L=ch_bubble_g 
   monster->direction = DIR_L;
   monster->frames.setDirection(DIR_L);

   fTgs = 60;        // proc tu byla 100?
   monster->frames.setChain(CHAIN_BUBBLE);
   frames.setChain(CHAIN_STAND);
   jumpNext();
}

//===========================================================================
int Bubble::Blow(Player *thissender)
{
   int result;
   int bc;
   byte bi;
   Bonus *pom_bo;
   
   //kdo sezere body?
   if (thissender!=NULL)
   {
      sender=thissender;
   }
   
   result = 0;

	// pokud uvnitr neco bylo, musime se o tom postarat!
   if (monster != NULL)
   {
      sound_device->Play(39,newx+20);
      if (sender && monster->type == MONSTER)
      {
			// bublinu prasknul hrac
			// takze bude bonus
         bc = random(((Monster *)monster)->bonus_cr)+1;
         if(bc) for(int k=0;k<bc;k++)
         {
            //V: po uprave bude 'bi' kandidat na zruseni!
            bi = 40 + (BYTE)(random(7));
            pom_bo = new Bonus(newx,newy,properties.member[bi]);
            if (ARRAY_FULL(bonuses_ctrl))
            {
               delete pom_bo;
            }
            else
            {
               ARRAY_ADD(bonuses,bonuses_ctrl,pom_bo);
            }
         }
			// ..body za potvoru
         int cp = ((Monster *)monster)->catch_points;
         int kp = ((Monster *)monster)->kill_points;
         whose->sheet.AddPoints(cp);
         sender->sheet.AddPoints(kp);
			// ... a jeji likvidace
         delete (monster);
         monstercnt--;
         monster = NULL;
			result = 1;
      }
      else
      {
			// bublina praskla sama unavou =)
         monster->newx = monster->oldx = newx;
         monster->newy = monster->oldy = newy;
         monster->mul_x = mul_x;
         monster->mul_y = mul_y;
         monster->direction = mon_dir;
         monster->frames.setDirection(mon_dir);
         if (mon_chain != CHAIN_FIRE)
            monster->frames.setChain(mon_chain);
         else
            monster->frames.setChain(CHAIN_STAND);
         if (monster->type == PLAYER)
            ((Player *)monster)->state ^= INBUBBLE;
         monster->jumpNext();
         monster->Reincarnate();
         if (ARRAY_FULL(monsters_ctrl))
         {
            delete monster;
         }
         else
         {
            ARRAY_ADD(monsters,monsters_ctrl,monster);
         }
         monster = NULL;
      }
   }
   else
   {
		// uvnitr nic nebylo, jenom zahrajeme jiny zvuk
      sound_device->Play(4,newx+20);
   }

	// zbyva jen puknuti samotne bubliny
   counter = BUB_BLOWTIME;
   frames.setChain(CHAIN_BLOW);
   fTgs = 15;
   state |= BLOWING;
   if (s_drops) drops->Blast(newx+10,newy+10,dropcolor,gravitation);
   //dominovy efekt
   Domino();

   return result;
}

//##########################################################################
//------------------------------ SymbolBubble ------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// vykresleni bubliny
//-------------------------------------------------------------------------
void SymbolBubble::Paint()
{
	// trochu prasarna - velikost grafiky symbolu i bublifuku v pixelech
	#define SYMSIZE 128

   Texture mask = NULL;

	RECT tr;
	tr.top = newy;
	tr.left = newx;
	tr.bottom = newy + ysize;
	tr.right = newx + xsize;
	tr = dx8.Coord(tr);

   if (id != ID_SBUBBLE)
   {
		// symbol v bublin
      RECT r;
		// maska me bt grafika bublifuku
      if (this->mask == NULL)
      {
			// neni, take bude njak symbol
         if (frames.chainid == CHAIN_STAND)
         {
            mask = sym_bubgfx;
            r.left = symbol*SYMSIZE;
         }
      }
      else
      {
			// take bublifuk
         mask = this->mask;
         r.left = 0;
      }

		// vykresleni masky - symbolu nebo bublifuku
      if (mask != NULL)
      {
         r.right = r.left+SYMSIZE;
         r.top = 0;
         r.bottom = SYMSIZE;
			dx8.BltSprite(&tr, mask, &r, 0);
      }

		// bublina
		// podle druhu symbolu si nastavm odstn bubliny
		int bubmask = 0xffffffff;
		if (symbol < 2)
		{
			if (symbol == 0) bubmask = 0xffffff80;
			else bubmask = 0xffff8080;
		}
		else
		{
			if (symbol == 2) bubmask = 0xff80ff80;
			else bubmask = 0xff80a0ff;
		}

		dx8.BltSprite(&tr, gfx, &frameRect, 0,bubmask);
      oldx = newx;
      oldy = newy;
      return;
   }

}

//-------------------------------------------------------------------------
// posun frejmu ve framechainu
//-------------------------------------------------------------------------
void SymbolBubble::jumpNext(int steps)
{
   frames.jumpNext();
   if (id != ID_SBUBBLE)
   {
      frameLeft = (frames.frameid % celr) * celw;
      frameTop = (frames.frameid / celr) * celh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + celw;  //xsize
      frameRect.bottom = frameTop + celh;  //ysize
	}
}

//===========================================================================
int SymbolBubble::Blow(Player *thissender)
{
   int result;
   //kdo sezere body? (pro domino)
   if (thissender!=NULL)
   {
      sender=thissender;
   }
   
   result = 0;
   sound_device->Play(39,newx+20);
   if (sender)
   {
      sender->symbuf.Add(symbol);
      sender->symbuf.Realize(sender);
   }
   counter = BUB_BLOWTIME;
   frames.setChain(CHAIN_BLOW);
   fTgs = 15;
   state |= BLOWING;
   if (s_drops) drops->Blast(newx+10,newy+10,DROP_WHITE,gravitation);
   //dominovy efekt
   Domino();
   
   return result;
}
