/*-------------------------------------------------------------------------

  Texture8Manager
  by Gunny & Vojvo (kind of)
      
  Spravuje textury a tm pdem i surfaces. 
  Kazda textura je prirazena spolu se svym jmenem 
  - pokud nejde o jmeno souboru, mela by byt prvnim znakem jmena hvezdicka
  (Ale to je jenom pomocn rozlien, kter by se mlo oetit zvenku,
   jinak je podstatn parametr TextureType.)

  Pokud chce uivatel uetit pam, me texturm vnutit 16bpp
  i u 32bitovch obrzk (R24+) nastavenm promnn force16bit.

   - 22.6.2002
     do textureex se ukld i pointer na pointer na texturu,
     kter pouv "extern" aplikace (t.j. mimo texmanagera)
     pi releasu textur se tam chmtne a nastav se tam null 
     (null tam je defaultn, je to teba nastavit zvenku!)
   * 2.3.2003
     restore automaticky aktualizuje dependent pointery

  stav:
   u to vypad slun

  todo:
   ?

   3. 3.2003 / upraveny Load a Create (ptex, automaticky nastavuji dependenty)
               releaseall kompletn likviduje COMPUTED textury (vcetne texex)
   2. 3.2003 / restoreall automaticky aktualizuje i extern dependency
               opraveny bug v remove (dusledek zmeny struktury textureEx)
  28.10.2002 / obnovena globln promnn textureManager
               (pln se v kontruktoru)
  16. 8.2002 / defaultn se nastavuje directory na "data\\"
               pidno getTexEx
  22. 6.2002 / pidna monost navsit na texturu pointer z venku 
               a jistit si tak kontrolu jejho stavu
  11. 6.2002 / restorovn odremovno, ale nemlo by se zvenku volat
               pidna funkce create
  10. 6.2002 / cel restorovn textur kvli devicelost je zaremovan,
               protoe vechny textury jsou MANAGED a je to zbyten
  12. 5.2002 / upraveno pro regulrn pouvan force16
  11. 3.2002 / deleteall
  02.02.2002 / restoreall
  27.01.2002 / releaseall
  26.01.2002 / zaremovan release a create
  14.07.2001 / na troskch surfacemanagera2 (hrub pekopvka)

        
-------------------------------------------------------------------------*/

#include <string.h>
#include <stdio.h>
#include "dx8graph.h"

#include "fakedir.h"
#include "exceptions.h"
#include "utils.h"
#include "tex8manager.h"

TextureManager *textureManager = NULL;
extern const char *glSpritesFake = "data\\sprites.dir";
extern const char *glMiscFake = "data\\misc.dir";

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
TextureManager::TextureManager()
{
   count = 0;
   directory = NULL;
   force16bit = 0;
   SetDirectory("data\\");
   //
   textureManager = this;
}

//-------------------------------------------------------------------------
// nastavuje adresar pro nacitani souboru
//-------------------------------------------------------------------------
void TextureManager::SetDirectory(const char *dir)
{
   if (directory != NULL)
   {
      delete [] directory;
      directory = NULL;
   }
   directory = new char[strlen(dir)+2];
   directory[0] = '\x0';
   strcpy(directory,dir);
   int len = strlen(directory);
   if (len > 0 && directory[len-1] != '\\')
      strcat(directory,"\\");
}

//-------------------------------------------------------------------------
// prida zadanou texturu pod specifikovanym jmenem
//-------------------------------------------------------------------------
void TextureManager::Add(Texture tex, Texture * ptex,
                         const char *name, const char *fakename, 
                         RFileHeader *header, 
                         TextureType stype, 
                         bool can_force_16bpp,
                         bool useDirectory )
{
   int i = 0;
   
   while (i < count)
   {
      if (!strcmp(textures[i].szName,name))
      {
         if (tex != textures[i].lpTexture)
         {
            sprintf(exc_string,"TextureManager.add(%s): Textura ji existuje!",name);
            throw new Exception(exc_string);
         }
         else
            return;
      }
      i++;
   }
   
   if (count < MAX_TEXTURES)
   {
      if (name != NULL)
      {
         textures[count].szName = new char[strlen(name)+1];
         strcpy(textures[count].szName,name);
      }
      else
         textures[count].szName = NULL;
      if (fakename != NULL)
      {
         textures[count].szFakename = new char[strlen(fakename)+1];   
         strcpy(textures[count].szFakename,fakename);
      }
      else
         textures[count].szFakename = NULL;

      textures[count].eType = stype;
      textures[count].lpTexture = tex;
      textures[count].lpHeader = header;
      textures[count].bUseDirectory = useDirectory;
      textures[count].bCanForce16 = can_force_16bpp;
      textures[count].dependentTexture = ptex;
      count++;
   }
   else
      throw new Exception("TextureManager.add: Pli mnoho textur");
}

//-------------------------------------------------------------------------
// vytvori novou prazdnou texturu a prida jej do manageru
//-------------------------------------------------------------------------
void TextureManager::Create(Texture * ptex, DWORD _width, DWORD _height, 
                               const char *_name, D3DFORMAT _format = D3DFMT_A8R8G8B8)
{
   Texture tex = NULL;
   // je treba nastavit rozmery textury (nejblizsi vetsi mocninu 2)
   DWORD p2width = powerof2(_width);
   DWORD p2height = powerof2(_height);
   ddResult = dx8.lpd3dDevice->CreateTexture(p2width, p2height, 1, 0, _format, D3DPOOL_MANAGED, &tex);
   if (ddResult != D3D_OK)
   {
      throw new DDException("LoadRFile: CreateTexture() chyba!",ddResult);
   }

   Add(tex, ptex, _name, NULL, NULL, TX_COMPUTED);
   *ptex = tex;
   return;
}

//-------------------------------------------------------------------------
// vymae vechny textury (na konci prce)
//-------------------------------------------------------------------------
void TextureManager::DeleteAll()
{
   int i = 0;
   ULONG u;
   
   while (i < count)
   {
      if (textures[i].lpTexture != NULL)
      {
         u = -1;
         while (u!=0)
         {
            u = textures[i].lpTexture->Release();
         }
         textures[i].lpTexture = NULL;
         // vymazu externi odkaz na texturu
         // samozrejme ten odkaz musi byt jeste platny!
	// ...co ale bohuel odtud nen jak ovit...
	// (mu mt nenullov pointer do zlikvidovanho objektu)
	// ili tohle je dost problematick trik
	// mon by to tady bylo lep vbec nedlat
	// ***
         if (textures[i].dependentTexture != NULL) 
            *(textures[i].dependentTexture) = NULL;
      }
      if (textures[i].szName != NULL)
      {
         delete textures[i].szName;
         textures[i].szName = NULL;
      }
      if (textures[i].szFakename != NULL)
      {
         delete textures[i].szFakename;
         textures[i].szFakename = NULL;
      }
      if (textures[i].lpHeader != NULL)
      {
         delete textures[i].lpHeader;
         textures[i].lpHeader = NULL;
      }
      i++;
   }
}
                         
//-------------------------------------------------------------------------
// uvoln vechny textury (nap. po devicelost), aby se daly znovu nast
// u FROMFILE nech texex, aby se daly obnovit
// COMPUTED textury kompletne smaze
//-------------------------------------------------------------------------

void TextureManager::ReleaseAll()
{
   int i = 0;
   
   while (i < count)
   {
      if (textures[i].lpTexture != NULL)
      {
         // uvolneni vlastni textury
         textures[i].lpTexture->Release();
         textures[i].lpTexture = NULL;
         if (textures[i].dependentTexture != NULL) 
            *(textures[i].dependentTexture) = NULL;
         
         // COMPUTED textury se muzou zahodit komplet
         if (textures[i].eType == TX_COMPUTED)
         {
            // zlikviduju alokovane veci z texex
            if (textures[i].szName != NULL)
               delete textures[i].szName;
            if (textures[i].szFakename != NULL)
               delete textures[i].szFakename;
            if (textures[i].lpHeader != NULL)
               delete textures[i].lpHeader;
            // a premaznu poslednim texexem
            textures[i] = textures[count-1];
            // takze tenhle slot musime vzit jeste jednou
            i--;
            count--;
         }
      }
      i++;
   }
}
                         
//-------------------------------------------------------------------------
// obnov vechny (FROMFILE) textury (nap. po devicelost) natenm ze soubor
//-------------------------------------------------------------------------

void TextureManager::RestoreAll()
{
   int i = 0;
   
   while (i < count)
   {
      // je textura fakt vymazana?
      if (textures[i].lpTexture == NULL)
      {
         // jake bpp muze textura aktualne pouzit?
         int bpp_limit = 32;
         if (textures[i].bCanForce16 && force16bit) bpp_limit = 16;
         
         // load
         // melo by se kontrolovat, jestli to je textura ze souboru
         // (at se marne nesnazim nacist generovanou)
         if (textures[i].eType == TX_FROMFILE)
         {
            // use directory?
            if (textures[i].bUseDirectory && directory != NULL)
            {
               strcpy(pom_string,directory);
            }
            else
               strcpy(pom_string,"");
            strcat(pom_string,textures[i].szName);
            textures[i].lpTexture = LoadRFile(pom_string, textures[i].szFakename,textures[i].lpHeader, bpp_limit);
        if (textures[i].dependentTexture != NULL) 
            *(textures[i].dependentTexture) = textures[i].lpTexture;
         }
      }
      i++;
   }
}
                         
//-------------------------------------------------------------------------
// nastavi jmeno pro zadanou texturu
//-------------------------------------------------------------------------
void TextureManager::SetName(Texture tex, const char *name)
{
   int i = 0;
   
   while (i < count)
   {
      if (textures[i].lpTexture == tex)
      {
         delete [] textures[i].szName;
         textures[i].szName = new char[strlen(name)+1];
         strcpy(textures[i].szName,name);
         return;
      }
      i++;
   }   
   return;
}


//-------------------------------------------------------------------------
// vrati texturu se zadanym jmenem
//-------------------------------------------------------------------------
Texture TextureManager::Get(const char *_name)
{
   int i = 0;
   
   while (i < count)
   {
      if (!stricmp(textures[i].szName,_name))
         return textures[i].lpTexture;
      i++;
   }   
   return NULL;
}

//-------------------------------------------------------------------------
// vrati pointer na textureEx ("deskriptor")
//-------------------------------------------------------------------------
TextureEx * TextureManager::GetTexEx(const Texture tex)
{
   int i = 0;
   
   while (i < count)
   {
      if (textures[i].lpTexture == tex)
         return &(textures[i]);
      i++;
   }   
   return NULL;
}

//-------------------------------------------------------------------------
// vytvori novou texturu se zadanym jmenem a nahraje ji ze stejnojmenneho
// souboru
//-------------------------------------------------------------------------
void TextureManager::Load(Texture * ptex, const char *name, const char *fakename, 
                             bool can_force_16bpp, bool useDirectory)
{
   Texture tex;
   RFileHeader *header = new RFileHeader();
   //char pom_string[160]; // melo by byt definovane v utils
   int i = 0;
   
   while (i < count)
   {
      if (!stricmp(textures[i].szName,name))
      {
         *ptex = textures[i].lpTexture;
      }
      i++;
   }
   
   // jake bpp muze textura aktualne pouzit?
   int bpp_limit = 32;
   if (can_force_16bpp && force16bit) bpp_limit = 16;

   // load
   if (useDirectory && directory != NULL)
   {
      strcpy(pom_string,directory);
      strcat(pom_string,name);
      tex = LoadRFile(pom_string,fakename,header, bpp_limit);
   }
   else
   {
      tex = LoadRFile(name,fakename,header, bpp_limit);
   }
   Add(tex, ptex, name,fakename,header,TX_FROMFILE, can_force_16bpp, useDirectory);
   *ptex = tex;
   return;
}

//-------------------------------------------------------------------------
// odstrani zadanou texturu z manageru
//-------------------------------------------------------------------------    
void TextureManager::Remove(Texture tex, bool release)
{
   int i = 0;
   
   if (tex == NULL)
      return;
   while (i < count)
   {
      if (textures[i].lpTexture == tex)
      {
			// nasli jsme stejny pointer
         if (release)
         {
				// ma se uvolnit z pameti
            if (textures[i].lpTexture != NULL)
               textures[i].lpTexture->Release();
				// pokud sem smroval pointer z venku, tak se vynuluje
				// TOHLE je potenciln problm!
				// ten pointer me bt soust objektu, kter v okamiku mazn u neexistuje
				// takze pak bych prepisoval cizi pamet
            if (textures[i].dependentTexture != NULL) 
               *(textures[i].dependentTexture) = NULL;
         }
         if (textures[i].szName != NULL)
			{
            delete textures[i].szName;
				//textures[i].szName = NULL; // asi neni nutne
			}
         if (textures[i].szFakename != NULL)
			{
            delete textures[i].szFakename;
				//textures[i].szFakename = NULL;
			}
         if (textures[i].lpHeader != NULL)
			{
            delete textures[i].lpHeader;
				//textures[i].lpHeader = NULL;
			}

         // opraven Vojvobug
			// pokud to neni posledni polozka, posunu sem posledn texturu ze seznamu
         if (i < count-1) textures[i] = textures[count-1];	// doufam, ze to funguje i s objekty!
         /*
         textures[i].szName = textures[count-1].szName;
         textures[i].szFakename = textures[count-1].szFakename;
         textures[i].lpHeader = textures[count-1].lpHeader;
         */
         count--;
         return;
      }
      i++;
   }
}

//-------------------------------------------------------------------------
// odstrani texturu se zadanym jmenem z manageru
//-------------------------------------------------------------------------
void TextureManager::Remove(const char *name, bool release)
{
   int i = 0;
   
   while (i < count)
   {
      if (!stricmp(textures[i].szName,name))
      {
         if (release)
         {
            if (textures[i].lpTexture != NULL)
               textures[i].lpTexture->Release();
            if (textures[i].dependentTexture != NULL) 
               *(textures[i].dependentTexture) = NULL;
         }
         if (textures[i].szName != NULL)
            delete textures[i].szName;
         if (textures[i].szFakename != NULL)
            delete textures[i].szFakename;
         if (textures[i].lpHeader != NULL)
            delete textures[i].lpHeader;

         // opravena rafinovan Vojvova konstrukce (nepotalo se se zmnou struktury)
         textures[i] = textures[count-1];
         /*
         textures[i].szName = textures[count-1].szName;
         textures[i].szFakename = textures[count-1].szFakename;
         textures[i].lpHeader = textures[count-1].lpHeader;
         */
         count--;
         return;
      }
      i++;
   }
}

//-------------------------------------------------------------------------
// vraci ukazatel na nactenou hlavicku souboru s texturou
//-------------------------------------------------------------------------
RFileHeader * TextureManager::GetHeader(Texture tex)
{
   int i=0;
   
   while (i<count)
   {
      if (textures[i].lpTexture == tex)
      {
         return textures[i].lpHeader;
      }
      i++;
   }
   
   return NULL;
}

//-------------------------------------------------------------------------
// vraci ukazatel na nactenou hlavicku souboru s texturou
//-------------------------------------------------------------------------
RFileHeader * TextureManager::GetHeader(const char *name)
{
   int i=0;
   
   while (i<count)
   {
      if (!strcmp(textures[i].szName,name))
      {
         return textures[i].lpHeader;
      }
      i++;
   }
   
   return NULL;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
TextureManager::~TextureManager()
{
   int i = 0;
   
   if (directory != NULL)
   {
      delete [] directory;
      directory = NULL;
   }
   
   while (i < count)
   {
      if (textures[i].lpTexture != NULL)
      {
         textures[i].lpTexture->Release();
         textures[i].lpTexture = NULL;
         if (textures[i].dependentTexture != NULL) 
            *(textures[i].dependentTexture) = NULL;
      }
      if (textures[i].szName != NULL)
         delete textures[i].szName;
      if (textures[i].szFakename != NULL)
         delete textures[i].szFakename;
      if (textures[i].lpHeader != NULL)
         delete textures[i].lpHeader;
      i++;
   }
   count = 0;
}
