/*--------------------------------------------------------------------
 TextureManager pro DX8
 *--------------------------------------------------------------------
 */

#ifndef TEX8MANAGER_H
#define TEX8MANAGER_H

#include "types8.h"	
#include "rfile32.h"	// mla by to bt verze s podporou alfa kanl!

#define MAX_TEXTURES 100

enum TextureType { TX_FROMFILE, TX_COMPUTED };

struct TextureEx {
   TextureType eType;   // TX_FROMFILE || TX_GENERATED
   Texture lpTexture;
   Name szName;
   Name szFakename;
   RFileHeader *lpHeader;
   bool bUseDirectory;
   bool bCanForce16;     // can force 16 bpp
   Texture * dependentTexture;
};

class TextureManager {
   TextureEx textures[MAX_TEXTURES];
   int count;
   
public:
   char *directory;
   char force16bit;

   TextureManager();
   ~TextureManager();

   // adds surface to the manager
   void Add(
      Texture tex,      // pridavana textura
      Texture * ptex,   // pointer na "externi" promennou s texturou (dependent)
      const char *name, const char *fakename, 
      RFileHeader *header = NULL, 
      TextureType ttype = TX_FROMFILE,
      bool can_force_16bpp = true,
      bool useDirectory = true);

   // creates 24-bit (well...32 RGBA...?)
   void Create(Texture * ptex, DWORD _width, DWORD _height, const char *_name, D3DFORMAT _format);
   
   // returns texture registered under given name
   Texture Get(const char *_name);

   // returns pointer to textureex
   TextureEx * GetTexEx(const Texture tex);
   
   // returns header of the texture registered under specified name
   RFileHeader* GetHeader(const char *name);

   // returns header of given surface
   RFileHeader* GetHeader(Texture tex);

   // loads a Texture from file and adds it to the manager
   void Load(Texture * ptex, const char *name, const char *fakename,
      bool can_force_16bpp = true, bool useDirectory = true);
   
   // uvoln vechny textury (nap. po devicelost)
   void ReleaseAll();

   // obnov vechny textury 
   void RestoreAll();
   
   // removes the Texture with given name from the manager
   void Remove(const char *name, bool release = true);

   // removes given Texture from the manager
   void Remove(Texture tex, bool release = true);

   // restores all textures and reloads those loaded from file
   //void restore();

   // assigns default directory for files
   void SetDirectory(const char *dir);

   // changes name of the Texture
   void SetName(Texture tex, const char *name);

   // vymazn vech textur - de facto destruktor
   void DeleteAll();

};

extern TextureManager *textureManager;
extern const char *glSpritesFake;
extern const char *glMiscFake;
 

#endif